import os
import json
from nektar import AppBase
from arkivist import Arkivist
from sometime import Sometime


def main():

    dt = Sometime().custom("%Y.%m.%d")

    counter = 0
    print("Initializing tests...")
    path = os.path.expanduser("~").replace("\\", "/")
    tests = Arkivist(path + "/nektar-tests.json", autosave=False)

    ##################################################
    # AppBase Class                                  #
    ##################################################
    
    custom_nodes = ["testnet.openhive.network"]
    TESTNET = "18dcf0a285365fc58b71f18b3d3fec954aa0c141c44e4e5cb4cf777b9eab274e"

    counter += 1
    print(f"\nTEST #{counter}: Initialize AppBase Class.")
    hive = AppBase()

    """
    counter += 1
    print(f"\nTEST #{counter}: Initialize AppBase Class with custom parameters.")
    custom_nodes = ["testnet.openhive.network"]
    hive = AppBase(nodes=custom_nodes, chain_id=TESTNET, timeout=5, retries=1, warning=True)

    counter += 1
    print(f"\nTEST #{counter}: Change to custom nodes.")
    hive.custom_nodes(custom_nodes)

    counter += 1
    print(f"\nTEST #{counter}: Add new WIF and equivalent role.")
    hive.append_wif(wif=tests["wifs0"]["posting"], role="posting")
    hive.append_wif(wif=tests["wifs0"]["active"], role="active")

    counter += 1
    print(f"\nTEST #{counter}: Add new dictionary of roles and their equivalent WIFs.")
    hive.append_wif(tests["wifs0"])  # {"posting": "5*", "active": "5*"}

    counter += 1
    print(f"\nTEST #{counter}: Change retries to custom value.")
    hive.set_timeout(15)

    ## Read the actual Hive API documentation for proper guidance
    ## https://developers.hive.io/apidefinitions/

    counter += 1
    print(f"\nTEST #{counter}: Access API and its methods - generic")
    hive.api("condenser")
    hive.api("condenser_api")

    counter += 1
    print(f"\nTEST #{counter}: `account_by_key_api.get_key_references`")
    # version = hive.api("account_by_key_api").*
    params = {"keys": [tests["publickey0"]]}
    data = hive.account_by_key().get_key_references(params)
    print("Account by public key: " + json.dumps(data, indent=2))

    counter += 1
    print(f"\nTEST #{counter}: `bridge.get_profile`")
    # version = hive.api("bridge").*
    params = {"account": tests["username0"], "observer": tests["username0"]}
    data = hive.bridge().get_profile(params)
    print("Profile: " + json.dumps(data, indent=2))

    counter += 1
    print(f"\nTEST #{counter}: `account_history_api.get_account_history`")
    # version = hive.api("account_history").*
    params = {
        "account": tests["username1"],
        "start": 1000,
        "limit": 1000,
        "include_reversible": True,
        "operation_filter_low": 0,
        "operation_filter_high": 1,
    }
    data = hive.account_history().get_account_history(params)
    print("Account history: " + json.dumps(data, indent=2))

    counter += 1
    print(f"\nTEST #{counter}: `block_api.get_block`")
    params = {"block_num": 8675309}
    data = hive.block().get_block(params)
    print("Block: " + json.dumps(data, indent=2))

    counter += 1
    print(f"\nTEST #{counter}: Create an instance for specific API only.")
    condenser = hive.condenser()

    params = [tests["username1"], tests["permlink1"]]
    data = condenser.get_content(params)
    print("Content #1: " + json.dumps(data, indent=2))

    params = [tests["username1"], 0, 10]
    data = condenser.get_blog(params)
    print("Content #2: " + json.dumps(data, indent=2))

    counter += 1
    print(f"\nTEST #{counter}: Using the `request` method.")
    method = "rc_api.find_rc_accounts"
    params = {"accounts": [tests["username1"]]}
    data = hive.request(method, params, strict=False, mock=True)
    print("Accounts: " + json.dumps(data, indent=2))
    """

    counter += 1
    print(f"\nTEST #{counter}: Using the `broadcast` method - `condenser_api`.")
    hive.append_wif(wif=tests["wif1"], role="posting")
    method = "condenser_api.get_transaction_hex"
    transaction = {
        "ref_block_num": 27033,
        "ref_block_prefix": 1234340064,
        "expiration": "2022-11-11T6:15:00",
        "operations": [
            [
                "vote",
                {
                    "voter": tests["username1"],
                    "author": tests["username3"],
                    "permlink": tests["permlink3"],
                    "weight": 10000,
                },
            ]
        ],
        "extensions": [],
    }
    data = hive.broadcast(method, transaction, strict=False, mock=True)
    print(data)
    print("Transaction hex: " + data)


if __name__ == "__main__":
    main()



import os
import json
from arkivist import Arkivist
from nektar import Waggle, Swarm
from sometime import Sometime

def main():

    dt = Sometime().custom("%Y.%m.%d")

    counter = 0
    print("Initializing tests...")
    path = os.path.expanduser("~").replace("\\", "/")
    tests = Arkivist(path + "/nektar-tests.json", autosave=False)

    ##################################################
    # Drone Class                                    #
    # Custom JSON methods                            #
    ##################################################
    
    custom_nodes = ["testnet.openhive.network"]
    TESTNET = "18dcf0a285365fc58b71f18b3d3fec954aa0c141c44e4e5cb4cf777b9eab274e"


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
    nektar.utils
    ~~~~~~~~~

    :copyright: 2022 Rodney Maniego Jr.
    :license: MIT License
"""


import time
from re import findall
from datetime import datetime, timezone
from .constants import ROLES, DATETIME_FORMAT


class NektarException(Exception):
    """ """

    def __init__(self, message):
        super().__init__(message)


def check_wifs(roles, operation):
    """Check if supplied WIF is in the required authority for the specific operation.

    Parameters
    ----------
    roles : list
        list of key authority
    operation : str
        operation name

    Returns
    -------
        Boolean value.
    """
    return bool(len([r for r in ROLES[operation] if r in roles]))


def make_expiration(seconds=30, formatting=None):
    """Return a UTC datetime formatted for the blockchain.

    Parameters
    ----------
    seconds : int
        seconds to add to the current timestamp (Default is 30)
    formatting : str, None
        format of the date-time string

    Returns
    -------
    str:
        Formatted date and time string.
    """
    timestamp = time.time() + int(seconds)
    if formatting is None:
        formatting = DATETIME_FORMAT
    valid_string(formatting)
    return datetime.utcfromtimestamp(timestamp).strftime(formatting)


def valid_string(value, pattern=None, fallback=None):
    """Check if the value is a valid string.

    Parameters
    ----------
    value : str
        value to be tested
    pattern : str
        regex pattern
    fallback : str, None
        value if failing

    Returns
    -------
    str:
        The value or the fallback.
    """
    if not isinstance(value, str):
        if fallback is None:
            raise NektarException("The value must be a string.")
        return fallback
    if pattern is None:
        return value
    if not bool(len(pattern.findall(value))):
        raise NektarException("The value is unsupported.")
    return value


def greater_than(value, minimum):
    """Check if input is greater than, otherwise return fallback.

    Parameters
    ----------
    value :
        value to be tested
    minimum :
        value to be tested against

    Returns
    -------
        The value or the fallback.

    """
    if not (isinstance(value, int) or value > minimum):
        raise ValueError(f"Value must be an integer greater than {minimum}.")


def within_range(value, minimum, maximum):
    """Check if input is within the range, otherwise return fallback.

    Parameters
    ----------
    value :
        value to be tested
    minimum :
        minimum value of the range
    maximum :
        maximum value of the range

    Returns
    -------

    """
    if not (isinstance(value, int) and (minimum <= int(value) <= maximum)):
        raise ValueError(f"Value must be within {minimum} to {maximum} only.")


def is_boolean(value):
    """Check if input is boolean, otherwise return fallback.

    Parameters
    ----------
    value :
        value to be tested

    Returns
    -------
        The value or the fallback.

    """
    if not isinstance(value, bool):
        raise TypeError("Value must be `True` or `False` only.")



import os
import json
from arkivist import Arkivist
from nektar import Waggle, Swarm
from sometime import Sometime

def main():

    dt = Sometime().custom("%Y.%m.%d")

    counter = 0
    print("Initializing tests...")
    path = os.path.expanduser("~").replace("\\", "/")
    tests = Arkivist(path + "/nektar-tests.json", autosave=False)

    ##################################################
    # Waggle Class                                   #
    # Methods for blogging and engagement            #
    ##################################################

    counter += 1
    print(f"\nTEST #{counter}: Initialize Waggle class.")
    hive = Waggle(tests["username1"])

    blogs = hive.blogs(limit=1)
    for blog in blogs:
        author = blog["author"]
        permlink = blog["permlink"]

        counter += 1
        print(
            f"\nTEST #{counter}: Get the content (blog or comment)."
        )
        content = hive.get_content(author, permlink)
        print(json.dumps(content, indent=2))

        counter += 1
        print(
            f"\nTEST #{counter}: Get the list replies in a blog post."
        )
        replies = hive.replies(author, permlink)
        print(json.dumps(replies, indent=2))

        counter += 1
        print(
            f"\nTEST #{counter}: Get the list of votes in a blog post."
        )
        votes = hive.votes(author, permlink)
        print(json.dumps(votes, indent=2))

        counter += 1
        print(
            f"\nTEST #{counter}: Returns a list of authors that have reblogged a post."
        )
        reblogs = hive.reblogs(author, permlink)
        print("Reblogged by: " + ", ".join(reblogs))


if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
"""
    nektar.condenser
    ~~~~~~~~~

    Interact with Hive API in Python.

    :copyright: 2022 Rodney Maniego Jr.
    :license: MIT License
"""

from .appbase import AppBase
from .constants import (
    NEKTAR_VERSION,
    BLOCKCHAIN_OPERATIONS,
    DISCUSSIONS_BY,
    ASSETS,
    ROLES,
    DATETIME_FORMAT,
    RE_PERMLINK,
    RE_COMMUNITY,
    RE_DATETIME,
)
from .utils import (
    check_wifs,
    make_expiration,
    valid_string,
    greater_than,
    within_range,
    is_boolean,
)


class Condenser:
    """Condenser class.
    ~~~~~~~~~

    Parameters
    ----------
    username :
        a valid Hive account username
    wifs :
        a dictionary of roles and their equivalent WIFs (Default is None)
    app :
        the name of the app built with nektar (Default is None)
    version :
        the version `x.y.x` of the app built with nektar (Default is None)
    timeout :
        seconds before the request is dropped (Default is 10)
    retries :
        times the request retries if errors are encountered (Default is 3)
    warning :
        display warning messages (Default is False)
    """

    def __init__(
        self,
        nodes=None,
        chain_id=None,
        timeout=10,
        retries=3,
        warning=False,
    ):
        self.appbase = AppBase(
            nodes=nodes,
            chain_id=chain_id,
            timeout=timeout,
            retries=retries,
            warning=warning,
        )
        self.api = self.appbase.condenser()

    def get_account_count(self):
        """Returns the number of accounts.
        https://developers.hive.io/apidefinitions/#condenser_api.get_account_count

        Returns
        -------
        int:
        """

        return self.api.get_account_count([])

    def get_account_history(self, account, start, limit, low=None, high=None):
        """Returns a history of all operations for a given account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_account_history https://gitlab.syncad.com/hive/hive/-/blob/master/libraries/protocol/include/hive/protocol/operations.hpp

        Parameters
        ----------
        account : str
            any valid Hive account username
        start : int
            starting range, or -1 for reverse history
        limit : int
            upperbound limit from 1 up to 1000
        low : int, optional
            operation id (Default is None)
        high : int, optional
            operation id (Default is None)

        Returns
        -------
        list:
        """

        valid_string(account)
        greater_than(start, -1)
        within_range(limit, 1, 1000)
        params = [account, start, limit]
        operations = list(range(len(BLOCKCHAIN_OPERATIONS)))
        if isinstance(low, int):
            ## for the first 64 blockchain operation
            if int(low) not in operations:
                raise ValueError(
                    "Operation Filter `low` is not a valid blockchain operation ID."
                )
            params.append(int("1".ljust(low + 1, "0"), 2))

        if isinstance(high, int):
            ## for the next 64 blockchain operation
            if int(high) not in operations:
                raise ValueError(
                    "Operation Filter `high` is not a valid blockchain operation ID."
                )
            params.append(0)  # set to `operation_filter_low` zero
            params.append(int("1".ljust(high + 1, "0"), 2))

        return self.api.get_account_history(params)

    def get_account_reputations(self, start, limit):
        """Returns a list of account reputations.
        https://developers.hive.io/apidefinitions/#condenser_api.get_account_reputations

        Parameters
        ----------
        start : str
            any valid Hive account username
        limit : int
            Maximum number of results, from 1 up to 1000.

        Returns
        -------
        list:
        """

        valid_string(start)
        within_range(limit, 1, 1000)
        params = [start, limit]
        return self.api.get_account_reputations(params)

    def get_accounts(self, accounts, delayed_votes_active):
        """Returns accounts, queried by name.
        https://developers.hive.io/apidefinitions/#condenser_api.get_accounts

        Parameters
        ----------
        accounts : list
            a list of any valid Hive account usernames
        delayed_votes_active : bool
            delayed votes hidden

        Returns
        -------
        list:
        """

        params = [accounts, delayed_votes_active]
        if not isinstance(accounts, list):
            raise TypeError("`accounts` must be a list of strings.")
        is_boolean(delayed_votes_active)
        return self.api.get_accounts(params)

    def get_active_votes(self, author, permlink):
        """Returns all votes for the given post.
        https://developers.hive.io/apidefinitions/#condenser_api.get_active_votes

        Parameters
        ----------
        author : str
            Author of the post.
        permlink : str
            Permlink of the post

        Returns
        -------
        list:
        """

        valid_string(author)
        valid_string(permlink, RE_PERMLINK)
        return self.api.get_active_votes([author, permlink])

    def get_active_witnesses(self):
        """Returns the list of active witnesses.
        https://developers.hive.io/apidefinitions/#condenser_api.get_active_witnesses

        Returns
        -------
        list:
        """

        return self.api.get_active_witnesses([])

    def get_block(self, number):
        """Returns a block.
        https://developers.hive.io/apidefinitions/#condenser_api.get_block

        Parameters
        ----------
        number : int
            block number

        Returns
        -------
        dict:
            Dictionary of block information.
        """

        greater_than(number, 0)
        return self.api.get_block([number])

    def get_block_header(self, number):
        """Returns a block header.
        https://developers.hive.io/apidefinitions/#condenser_api.get_block_header

        Parameters
        ----------
        number : int
            block number

        Returns
        -------
        dict:
            Dictionary of block header information.
        """

        greater_than(number, 0)
        return self.api.get_block_header([number])

    def get_blog(self, account, start, limit):
        """Returns the list of blog entries for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_blog

        Parameters
        ----------
        account : str
            any valid Hive account username
        start : int
            starting entry id
        limit : int
            maximum number of results 1 up to 500

        Returns
        -------
        list:
        """

        valid_string(account)
        start = greater_than(start, 0)
        limit = within_range(limit, 1, 500)
        params = [account, start, limit]
        return self.api.get_blog(params)

    def get_blog_authors(self, account):
        """Returns a list of authors that have had their content reblogged on a given blog account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_blog_authors

        Issue
        ---------
             Assert Exception:false: Supported by hivemind

        Parameters
        ----------
        account : str
            any valid Hive account username

        Returns
        -------
        dict:
            Dictionary of block header information.
        """

        valid_string(account)
        return self.api.get_blog_authors([account])

    def get_blog_entries(self, account, start, limit):
        """Returns a list of blog entries for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_blog_entries

        Parameters
        ----------
        account : str
            any valid Hive account username
        start : int
            starting entry id
        limit : int
            maximum number of results 1 up to 500

        Returns
        -------
        list:
            List of blogs and its basic information without the full blog data.
        """

        valid_string(account)
        start = greater_than(start, 0)
        limit = within_range(limit, 1, 500)
        params = [account, start, limit]
        return self.api.get_blog_entries(params)

    def get_chain_properties(self):
        """Returns the chain properties.
        https://developers.hive.io/apidefinitions/#condenser_api.get_chain_properties

        Returns
        -------
        dict:
        """

        return self.api.get_chain_properties([])

    def get_config(self):
        """Returns information about compile-time constants.
        https://developers.hive.io/apidefinitions/#condenser_api.get_config
        https://developers.hive.io/tutorials-recipes/understanding-configuration-values.html

        Returns
        -------
        dict:
            A dictionary blockchain configurations.
        """

        return self.api.get_config([])

    def get_content(self, author, permlink):
        """Returns the content of a post or comment.
        https://developers.hive.io/apidefinitions/#condenser_api.get_content

        Parameters
        ----------
        author : list
            Author of the post.
        permlink : str
            Permlink of the post

        Returns
        -------
        dict:
        """

        valid_string(author)
        valid_string(permlink)
        params = [author, permlink]
        return self.api.get_content(params)

    def get_content_replies(self, author, permlink):
        """Returns a list of replies.
        https://developers.hive.io/apidefinitions/#condenser_api.get_content_replies

        Parameters
        ----------
        author : list
            Author of the post.
        permlink : str
            Permlink of the post

        Returns
        -------
        list:
            List of replies to a post or comment.
        """

        valid_string(author)
        valid_string(permlink)
        params = [author, permlink]
        return self.api.get_content_replies(params)

    def get_conversion_requests(self, account):
        """Returns a list of conversion request.
        https://developers.hive.io/apidefinitions/#condenser_api.get_conversion_requests

        Parameters
        ----------
        account : str
            Any valid Hive account username.

        Returns
        -------
        list:
        """

        valid_string(account)
        return self.api.get_conversion_requests([account])

    def get_current_median_history_price(self):
        """Returns the median history price.
        https://developers.hive.io/apidefinitions/#condenser_api.get_current_median_history_price

        Returns
        -------
        dict:
        """

        return self.api.get_current_median_history_price([])

    def get_discussions(
        self,
        by,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
        comment=False,
    ):
        """Returns a list of discussions.

        Parameters
        ----------
        by : str
            Discussion type: `active`, `blog`, `cashout`, `children`, `created`, `hot`, `payout`, `promoted`, `trending`, OR `votes`.
        tag : str
            Any valid string
        limit : int
            Maximum number of results
        filter_tags : list, None
            List of valid tags
        select_authors : list, None
            List of valid account username
        select_tags : list, None
            List of valid tags
        truncate : int, optional
            Truncate body (0, 1)
        comment : bool, optional
            Comment discussion, default is False.

        Returns
        -------
        list:
            List of discussions.
        """

        valid_string(by)
        if by not in DISCUSSIONS_BY:
            raise ValueError("`by` must be a value in:", ",".join(DISCUSSIONS_BY))

        valid_string(tag)
        within_range(limit, 1, 500)
        within_range(truncate, 0, 1)

        # initial parameters
        data = {"tag": tag, "limit": limit, "truncate_body": truncate}

        # custom filters
        if not isinstance(filter_tags, list):
            if filter_tags is not None:
                raise TypeError("`filter_tags` must be a list.")
            filter_tags = []
        if filter_tags:
            data["filter_tags"] = filter_tags
        if not isinstance(select_authors, list):
            if select_authors is not None:
                raise TypeError("`select_authors` must be a list.")
            select_authors = []
        if select_authors:
            data["select_authors"] = select_authors
        if not isinstance(select_tags, list):
            if select_tags is not None:
                raise TypeError("`select_tags` must be a list.")
            select_tags = []
        if select_tags:
            data["select_tags"] = select_tags

        match by:
            case "active":
                return self.api.get_discussions_by_active([data])
            case "blog":
                return self.api.get_discussions_by_blog([data])
            case "cashout":
                return self.api.get_discussions_by_cashout([data])
            case "children":
                return self.api.get_discussions_by_children([data])
            case "created":
                return self.api.get_discussions_by_created([data])
            case "hot":
                return self.api.get_discussions_by_hot([data])
            case "payout":
                if comment:
                    return self.api.get_comment_discussions_by_payout([data])
                return self.api.get_post_discussions_by_payout([data])
            case "promoted":
                return self.api.get_discussions_by_promoted([data])
            case "trending":
                return self.api.get_discussions_by_trending([data])
            case "votes":
                return self.api.get_discussions_by_trending([data])

    def get_discussions_by_active(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions based on active.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_active

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
            List of discussions.
        """

        by = "active"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_discussions_by_blog(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions by blog.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_blog

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
            List of discussions.
        """

        by = "blog"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_discussions_by_cashout(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions by cashout.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_cashout

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
            List of discussions.
        """

        by = "cashout"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_discussions_by_children(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions by cashout.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_children

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
            List of discussions.
        """

        by = "children"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_discussions_by_created(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions by created
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_created

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
            List of discussions.
        """

        by = "created"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_discussions_by_hot(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions by hot.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_hot

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
            List of discussions.
        """

        by = "hot"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_discussions_by_promoted(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions by promoted
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_promoted

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
            List of discussions.
        """

        by = "promoted"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_discussions_by_trending(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions by trending.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_trending

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
            List of discussions.
        """

        by = "trending"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_discussions_by_votes(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions by trending.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_votes

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
            List of discussions.
        """

        by = "votes"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_comment_discussions_by_payout(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of discussions based on payout.
        https://developers.hive.io/apidefinitions/#condenser_api.get_comment_discussions_by_payout

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
        """

        by = "payout"
        comment = True
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate, comment
        )

    def get_post_discussions_by_payout(
        self,
        tag,
        limit,
        filter_tags=None,
        select_authors=None,
        select_tags=None,
        truncate=0,
    ):
        """Returns a list of post discussions by payout.
        https://developers.hive.io/apidefinitions/#condenser_api.get_post_discussions_by_payout

        Parameters
        ----------
        tag : str
            any valid string
        limit : int
            maximum number of results
        filter_tags : list, None
            list of valid tags
        select_authors : list, None
            list of valid account username
        select_tags : list, None
            list of valid tags
        truncate : int, optional
            truncate body (0, 1)

        Returns
        -------
        list:
        """

        by = "payout"
        return self.get_discussions(
            by, tag, limit, filter_tags, select_authors, select_tags, truncate
        )

    def get_discussions_by_author_before_date(self, author, permlink, date, limit):
        """Returns a list of discussions based on author before date.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_author_before_date

        Parameters
        ----------
        author : list
            Author of the post.
        permlink : str
            Permlink of the post
        date : str
            Date before e.g. 1970-01-01T00:00:00
        limit : int
            Maximum number of results.

        Returns
        -------
        list:
        """

        valid_string(author)
        valid_string(permlink)
        valid_string(date, RE_DATETIME)
        greater_than(limit, 0)
        params = [author, permlink, date, limit]
        return self.api.get_discussions_by_author_before_date(params)

    def get_discussions_by_comments(
        self,
        author,
        permlink,
        limit,
    ):
        """Returns a list of discussions by cashout.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_comments

        Parameters
        ----------
        author : str
            Start string for author name.
        permlink : str
            Start string for permlink.
        limit : int

        Returns
        -------
        list:
            List of discussions.
        """

        valid_string(author)
        # allow empty start permlink
        valid_string(permlink)
        within_range(limit, 1, 100)
        data = {"start_author": author, "start_permlink": permlink, "limit": limit}
        return self.api.get_discussions_by_comments([data])

    def get_discussions_by_feed(
        self,
        tag,
        author,
        permlink,
        limit,
    ):
        """Returns a list of discussions by feed.
        https://developers.hive.io/apidefinitions/#condenser_api.get_discussions_by_feed

        Parameters
        ----------
        tag : str
            any valid string
        author : str
            Start string for author name.
        permlink : str
            Start string for permlink.
        limit : int

        Returns
        -------
        list:
            List of discussions.
        """

        valid_string(tag)
        valid_string(author)
        # allow empty start permlink
        valid_string(permlink)
        within_range(limit, 1, 100)
        data = {
            "tag": tag,
            "start_author": author,
            "start_permlink": permlink,
            "limit": limit,
        }
        return self.api.get_discussions_by_feed([data])

    def get_dynamic_global_properties(self):
        """Returns the current dynamic global properties.
        https://developers.hive.io/apidefinitions/#condenser_api.get_dynamic_global_properties
        https://developers.hive.io/tutorials-recipes/understanding-dynamic-global-properties.html

        Returns
        -------
        dict:
        """

        return self.api.get_dynamic_global_properties([])

    def get_escrow(self, account, eid):
        """Returns the escrow for a certain account by id.
        https://developers.hive.io/apidefinitions/#condenser_api.get_escrow

        Parameters
        ----------
        account : str
            Any valid Hive account username.
        eid : int
            escrow id

        Returns
        -------
        null:
        """

        valid_string(account)
        greater_than(eid, -1)
        return self.api.get_escrow([account, eid])

    def get_expiring_vesting_delegations(self, account, after):
        """Returns the expiring vesting delegations for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_expiring_vesting_delegations

        Parameters
        ----------
        account : str
            Any valid Hive account username.
        after : str
            Valid blockchain timestamp, e.g. "2018-01-01T00:00:00"

        Returns
        -------
        list:
        """

        valid_string(account)
        valid_string(after, RE_DATETIME)
        return self.api.get_expiring_vesting_delegations([account, after])

    def get_feed(self, account, eid, limit):
        """Returns a list of items in an account’s feed.
        https://developers.hive.io/apidefinitions/#condenser_api.get_feed

        Parameters
        ----------
        account : str
            Any valid Hive account.
        eid : int
            Starting entry id.
        limit : int
            Maximum number of results, 1 up to 500.

        Returns
        -------
        list:
            List of posts and its corresponding information
        """

        valid_string(account)
        greater_than(eid, -1)
        greater_than(limit, 0)
        return self.api.get_feed([account, eid, limit])

    def get_feed_entries(self, account, eid, limit):
        """Returns a list of entries in an account’s feed.
        https://developers.hive.io/apidefinitions/#condenser_api.get_feed_entries

        Parameters
        ----------
        account : str
            Any valid Hive account.
        eid : int
            Starting entry id.
        limit : int
            Maximum number of results, 1 up to 500.

        Returns
        -------
        list:
            List of posts and its corresponding information
        """

        valid_string(account)
        greater_than(eid, -1)
        within_range(limit, 1, 500)
        return self.api.get_feed_entries([account, eid, limit])

    def get_feed_history(self):
        """Returns the history of price feed values
        https://developers.hive.io/apidefinitions/#condenser_api.get_feed_history

        Returns
        -------
        dict:
        """

        return self.api.get_feed_history([])

    def get_follow_count(self, account):
        """Returns the count of followers/following for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_follow_count

        Parameters
        ----------
        account : str
            Any valid Hive account.

        Returns
        -------
        dict:
        """

        valid_string(account)
        return self.api.get_follow_count([account])

    def get_followers(self, account, start, ftype, limit):
        """Returns the list of followers for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_followers

        Parameters
        ----------
        account : str
            Any valid Hive account.
        start : str
            Starting character/s of the follower usernames.
        ftype : str
            Follow type, `blog` or `ignore`.
        limit : int
            Maximum number of results from 1 up to 1000.

        Returns
        -------
        list:
        """

        valid_string(account)
        valid_string(start)
        valid_string(ftype)
        within_range(limit, 1, 1000)
        params = [account, start, ftype, limit]
        return self.api.get_followers(params)

    def get_following(self, account, start, ftype, limit):
        """Returns the list of accounts that are following an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_following

        Parameters
        ----------
        account : str
            Any valid Hive account.
        start : str
            Starting character/s of the follower usernames.
        ftype : str
            Follow type, `blog` or `ignore`.
        limit : int
            Maximum number of results from 1 up to 1000.

        Returns
        -------
        list:
        """

        valid_string(account)
        valid_string(start)
        valid_string(ftype)
        within_range(limit, 1, 1000)
        params = [account, start, ftype, limit]
        return self.api.get_following(params)

    def get_hardfork_version(self):
        """Returns the current hardfork version.
        https://developers.hive.io/apidefinitions/#condenser_api.get_hardfork_version

        Returns
        -------
        str:
        """

        return self.api.get_hardfork_version([])

    def get_key_references(self, keys):
        """Returns all accounts that have the key associated with their owner or active authorities.
        https://developers.hive.io/apidefinitions/#condenser_api.get_key_references

        Parameters
        ----------
        keys : str, list
            List of valid Hive public keys.

        Returns
        -------
        list:
        """

        params = [[keys]]
        if not isinstance(keys, str):
            if not isinstance(keys, list):
                raise TypeError("`keys` must be a list of strings.")
            if not all([0 for k in keys if not isinstance(k, str)]):
                raise TypeError("`keys` must be a list of strings.")
            params = [keys]
        return self.api.get_key_references(params)

    def get_market_history(self, seconds, start, end):
        """Returns the market history for the internal HBD:HIVE market.
        https://developers.hive.io/apidefinitions/#condenser_api.get_market_history

        Parameters
        ----------
        seconds : int
            bucket seconds: 15, 60 (1 minute), 300 (5 mins), 3600 (1 hour), 86400 (1 day)
        start : str
            Valid starting blockchain timestamp, e.g. "2017-01-01T00:00:00"
        end : str
            Valid ending blockchain timestamp, e.g. "2018-01-01T00:00:00"

        Returns
        -------
        list:
        """

        if int(seconds) not in (15, 60, 300, 3600, 86400):
            raise ValueError(
                "`seconds` is not supported, see get_market_history_buckets."
            )
        valid_string(start, RE_DATETIME)
        valid_string(end, RE_DATETIME)
        return self.api.get_market_history([seconds, start, end])

    def get_market_history_buckets(self):
        """Returns the bucket seconds being tracked by the plugin.
        https://developers.hive.io/apidefinitions/#condenser_api.get_market_history_buckets

        Returns
        -------
        list:
        """

        return self.api.get_market_history_buckets([])

    def get_next_scheduled_hardfork(self):
        """Returns the next scheduled hardfork.
        https://developers.hive.io/apidefinitions/#condenser_api.get_next_scheduled_hardfork

        Returns
        -------
        dict:
        """

        return self.api.get_next_scheduled_hardfork([])

    def get_open_orders(self, account):
        """Returns the open orders for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_open_orders

        Parameters
        ----------
        account : str
            Any valid Hive account.

        Returns
        -------
        list:
        """

        valid_string(account)
        return self.api.get_open_orders([account])

    def get_ops_in_block(self, number, virtual):
        """Returns the open orders for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_ops_in_block

        Parameters
        ----------
        number : int
            Block number.
        virtual : bool
            Virtual operation or not, `True` or `False`.

        Returns
        -------
        list:
        """

        greater_than(number, 1)
        is_boolean(virtual)
        return self.api.get_ops_in_block([number, virtual])

    def get_order_book(self, limit):
        """Returns the internal market order book.
        https://developers.hive.io/apidefinitions/#condenser_api.get_order_book

        Parameters
        ----------
        limit : int
            Maximum number of results, from 1 up to 500.

        Returns
        -------
        dict:
        """

        greater_than(limit, 1)
        return self.api.get_order_book([limit])

    def get_owner_history(self, account):
        """Returns the owner history of an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_owner_history

        Parameters
        ----------
        account : str
            any valid Hive account username

        Returns
        -------
        dict:
            Dictionary of block header information.
        """

        valid_string(account)
        return self.api.get_owner_history([account])

    def get_potential_signatures(self, transaction):
        """This method will return the set of all public keys that could possibly sign for a given transaction.
        https://developers.hive.io/apidefinitions/#condenser_api.get_potential_signatures

        Parameters
        ----------
        transaction : dict
            Any valid unsigned transaction.

        Returns
        -------
        list:
        """

        if not isinstance(transaction, dict):
            raise ValueError("`transaction` must be a dictionary.")
        return self.api.get_potential_signatures([transaction])

    def get_reblogged_by(self, author, permlink):
        """Returns a list of authors that have reblogged a post.
        https://developers.hive.io/apidefinitions/#condenser_api.get_reblogged_by

        Parameters
        ----------
        author : list
            Author of the post.
        permlink : str
            Permlink of the post

        Returns
        -------
        list:
        """

        valid_string(author)
        valid_string(permlink, RE_PERMLINK)
        return self.api.get_reblogged_by([author, permlink])

    def get_recent_trades(self, limit):
        """Returns the most recent trades for the internal HBD:HIVE market.
        https://developers.hive.io/apidefinitions/#condenser_api.get_recent_trades

        Parameters
        ----------
        limit : int
            Maximum number of results, from 1 up to 1000.

        Returns
        -------
        list:
        """

        within_range(limit, 1, 1000)
        return self.api.get_recent_trades([limit])

    def get_recovery_request(self, account):
        """Returns the recovery request for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_recovery_request

        Parameters
        ----------
        account : str
            Any valid Hive account username.

        Returns
        -------
        null:
        """

        valid_string(account)
        return self.api.get_recovery_request([account])

    def get_replies_by_last_update(self, author, permlink, limit):
        """Returns a list of replies by last update.
        https://developers.hive.io/apidefinitions/#condenser_api.get_replies_by_last_update

        Parameters
        ----------
        author : str
            Start pattern for the post author.
        permlink : str
            Start pattern for the post permlink.
        limit : int
            Maximum number of results, from 1 up to 100.

        Returns
        -------
        list:
        """

        valid_string(author)
        # allow empty permlink
        valid_string(permlink)
        within_range(limit, 1, 100)
        return self.api.get_replies_by_last_update([author, permlink, limit])

    def get_required_signatures(self, transaction, keys):
        """This API will take a partially signed transaction
        and a set of public keys that the owner has the ability
        to sign for and return the minimal subset of public keys
        that should add signatures to the transaction.
        https://developers.hive.io/apidefinitions/#condenser_api.get_required_signatures

        Parameters
        ----------
        account : dict
            any valid unsigned transaction.

        Returns
        -------
        list:
        """

        if not isinstance(transaction, dict):
            raise ValueError("`transaction` must be a dictionary.")
        if not isinstance(keys, list):
            raise ValueError("`keys` must be a list of strings.")
        if not all([0 for k in keys if not isinstance(k, str)]):
            raise ValueError("`keys` must be a list of strings.")
        return self.api.get_required_signatures([transaction, keys])

    def get_reward_fund(self, action):
        """Returns information about the current reward funds.
        https://developers.hive.io/apidefinitions/#condenser_api.get_reward_fund

        Parameters
        ----------
        action : str
            `post`

        Returns
        -------
        null:
        """

        valid_string(action)
        return self.api.get_reward_fund([action])

    def get_savings_withdraw_from(self, account):
        """Returns savings withdraw from an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_savings_withdraw_from

        Parameters
        ----------
        account : str
            Any valid Hive account username.

        Returns
        -------
        list:
        """

        valid_string(account)
        return self.api.get_savings_withdraw_from([account])

    def get_savings_withdraw_to(self, account):
        """Returns savings withdraw from an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_savings_withdraw_to

        Parameters
        ----------
        account : str
            Any valid Hive account username.

        Returns
        -------
        list:
        """

        valid_string(account)
        return self.api.get_savings_withdraw_to([account])

    def get_tags_used_by_author(self, account):
        """Returns a list of tags used by an author.
        https://developers.hive.io/apidefinitions/#condenser_api.get_tags_used_by_author

        Parameters
        ----------
        account : str
            Any valid Hive account username.

        Returns
        -------
        list:
        """

        valid_string(account)
        return self.api.get_tags_used_by_author([account])

    def get_ticker(self):
        """Returns the market ticker for the internal HBD:HIVE market.
        https://developers.hive.io/apidefinitions/#condenser_api.get_ticker

        Returns
        -------
        dict:
        """

        return self.api.get_ticker([])

    def get_trade_history(self, start, end, limit):
        """Returns the expiring vesting delegations for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_trade_history

        Parameters
        ----------
        start : str
            Valid starting blockchain timestamp, e.g. "2017-01-01T00:00:00"
        end : str
            Valid ending blockchain timestamp, e.g. "2018-01-01T00:00:00"
        limit : int
            Maximum number of results, from 1 up to 100.

        Returns
        -------
        list:
        """

        valid_string(start, RE_DATETIME)
        valid_string(end, RE_DATETIME)
        within_range(limit, 1, 1000)
        return self.api.get_trade_history([start, end, limit])

    def get_transaction(self, tid):
        """Returns the details of a transaction based on a transaction id.
        https://developers.hive.io/apidefinitions/#condenser_api.get_transaction

        Parameters
        ----------
        tid : str
            Any valid transaction id.

        Returns
        -------
        list:
        """

        valid_string(tid)
        return self.api.get_transaction([tid])

    def get_transaction_hex(self, transaction):
        """This API will take a partially signed transaction
        and a set of public keys that the owner has the ability
        to sign for and return the minimal subset of public keys
        that should add signatures to the transaction.
        https://developers.hive.io/apidefinitions/#condenser_api.get_transaction_hex

        Parameters
        ----------
        transaction : dict
            Any valid unsigned transaction.

        Returns
        -------
        str:
        """

        if not isinstance(transaction, dict):
            raise ValueError("`transaction` must be a dictionary.")
        return self.api.get_transaction_hex([transaction])

    def get_trending_tags(self, start, limit):
        """Returns the list of trending tags.
        https://developers.hive.io/apidefinitions/#condenser_api.get_trending_tags

        Parameters
        ----------
        start : str
            Starting pattern for tags.
        limit : int
            Maximum number of results, from 1 up to 100.

        Returns
        -------
        list:
        """

        valid_string(start)
        within_range(limit, 1, 100)
        return self.api.get_trending_tags([start, limit])

    def get_version(self):
        """Returns the versions of blockchain, hive, and FC.
        https://developers.hive.io/apidefinitions/#condenser_api.get_version

        Returns
        -------
        dict:
        """

        return self.api.get_version([])

    def get_vesting_delegations(self, account, start, limit):
        """Returns the vesting delegations by an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_vesting_delegations

        Parameters
        ----------
        account : str
            Any valid Hive account username, delegator.
        start : str
            Starting pattern for delegatee username.
        limit : int
            Maximum number of results, from 1 up to 1000.

        Returns
        -------
        list:
        """

        valid_string(account)
        valid_string(start)
        within_range(limit, 1, 100)
        return self.api.get_vesting_delegations([account, start, limit])

    def get_volume(self):
        """Returns the market volume for the past 24 hours.
        https://developers.hive.io/apidefinitions/#condenser_api.get_volume

        Returns
        -------
        dict:
        """

        return self.api.get_volume([])

    def get_withdraw_routes(self, account, route):
        """Returns the withdraw routes for an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_withdraw_routes

        Parameters
        ----------
        account : str
            Any valid Hive account username, delegator.
        rout : str
            `all`, `outgoing`, or `incoming

        Returns
        -------
        list:
        """

        valid_string(account)
        valid_string(route)
        routes = ("incoming", "outgoing", "all")
        if route not in routes:
            raise ValueError("`route` must be a value in:", ",".join(routes))
        return self.api.get_withdraw_routes([account, route])

    def get_witness_by_account(self, account):
        """Returns the witness of an account.
        https://developers.hive.io/apidefinitions/#condenser_api.get_witness_by_account

        Parameters
        ----------
        account : str
            Any valid Hive account username.

        Returns
        -------
        null:
        """

        valid_string(account)
        return self.api.get_witness_by_account([account])

    def get_witness_count(self):
        """Return the number of witnesses.
        https://developers.hive.io/apidefinitions/#condenser_api.get_witness_count

        Returns
        -------
        int:
        """

        return self.api.get_witness_count([])

    def get_witness_schedule(self):
        """Returns the current witness schedule.
        https://developers.hive.io/apidefinitions/#condenser_api.get_witness_schedule

        Returns
        -------
        list:
        """

        return self.api.get_witness_schedule([])

    def get_witnesses(self, indices):
        """Returns current witnesses.
        https://developers.hive.io/apidefinitions/#condenser_api.get_witnesses

        Parameters
        ----------
        indices : int, list
            List of integers, from 0 up to the current number of witnesses.

        Returns
        -------
        list:
        """

        params = [[indices]]
        if not isinstance(indices, int):
            if not isinstance(indices, list):
                raise ValueError("`indices` must be a list of integers.")
            if not all([0 for i in indices if not isinstance(i, int)]):
                raise ValueError("`indices` must be a list of integers.")
            params = [indices]

        return self.api.get_witnesses(params)

    def get_witnesses_by_vote(self, start, limit):
        """Returns current witnesses by vote.
        https://developers.hive.io/apidefinitions/#condenser_api.get_witnesses_by_vote

        Parameters
        ----------
        start : str
            Starting pattern for witness username.
        limit : int
            Maximum number of results, from 1 up to 1000.

        Returns
        -------
        list:
        """

        valid_string(start)
        within_range(limit, 1, 1000)
        return self.api.get_witnesses_by_vote([start, limit])

    def lookup_account_names(self, accounts, delayed_votes_active):
        """Returns accounts, queried by name.
        https://developers.hive.io/apidefinitions/#condenser_api.lookup_account_names

        Parameters
        ----------
        accounts : list
            a list of any valid Hive account usernames
        delayed_votes_active : bool
            delayed votes active

        Returns
        -------
        list:
        """

        params = [[accounts], delayed_votes_active]
        is_boolean(delayed_votes_active)
        if not isinstance(accounts, str):
            if not isinstance(accounts, list):
                raise ValueError("`accounts` must be a list of strings.")
            if not all([0 for k in accounts if not isinstance(k, str)]):
                raise ValueError("`accounts` must be a list of strings.")
            params[0] = accounts
        return self.api.lookup_account_names(params)

    def lookup_accounts(self, start, limit):
        """Looks up accounts starting with name.
        https://developers.hive.io/apidefinitions/#condenser_api.lookup_accounts

        Parameters
        ----------
        start : str
            Starting pattern for Hive account username.
        limit : int
            Maximum number of results, from 1 up to 1000.

        Returns
        -------
        list:
        """

        valid_string(start)
        within_range(limit, 1, 1000)
        return self.api.lookup_accounts([start, limit])

    def lookup_witness_accounts(self, start, limit):
        """Looks up witness accounts starting with name.
        https://developers.hive.io/apidefinitions/#condenser_api.lookup_witness_accounts

        Parameters
        ----------
        start : str
            Starting pattern for Hive account username.
        limit : int
            Maximum number of results, from 1 up to 1000.

        Returns
        -------
        list:
        """

        valid_string(start)
        within_range(limit, 1, 1000)
        return self.api.lookup_witness_accounts([start, limit])

    def find_proposals(self, pid):
        """Finds proposals by proposal id.
        https://developers.hive.io/apidefinitions/#condenser_api.find_proposals

        Parameters
        ----------
        pid : int
            proposal.id, not proposal.proposal_id

        Returns
        -------
        dict:
        """

        greater_than(pid, 0)
        return self.api.find_proposals([[pid]])

    def list_proposal_votes(self, start, limit, order, direction=None, status=None):
        """Returns all proposal votes, starting with the specified voter or proposal.id.
        https://developers.hive.io/apidefinitions/#condenser_api.list_proposal_votes

        Parameters
        ----------
        start : int, str
            proposal id, or account name voting for the proposal
        limit : int
            number of votes, 0-1000
        order: str
            `by_voter_proposal` - order by proposal voter
            `by_proposal_voter` - order by proposal.id
        direction : str, None
            `ascending` or `descending`
        status : str, None
            `all`, `inactive`, `active`, `expired`, or `votable`

        Returns
        -------
        list:
        """

        params = [[""], 1000, "by_voter_proposal", "ascending", "all"]

        if not isinstance(start, (str, int)):
            raise ValueError("`start` must be a voter acount name or proposal id.")
        params[0][0] = start

        params[1] = within_range(limit, 0, 1000)
        if order not in ("by_voter_proposal", "by_proposal_voter"):
            raise ValueError("`order` is not supported.")
        params[2] = order

        if direction is not None:
            if direction not in ("ascending", "descending"):
                raise ValueError("`direction` is not supported.")
            params[3] = direction

        if status is not None:
            statuses = ("all", "inactive", "active", "expired", "votable")
            if status not in statuses:
                raise ValueError("`status` is not supported.")
            params[4] = status

        return self.api.list_proposal_votes(params)

    def list_proposals(self, start, limit, order, direction=None, status=None):
        """Returns all proposals, starting with the specified creator or start date.
        https://developers.hive.io/apidefinitions/#condenser_api.list_proposals

        Parameters
        ----------
        start : int, str
            `creator` - creator of the proposal
            `start_date` - start date of the proposal, e.g. "2022-11-27T00:00:00"
            `end_date` - end date of the proposal, e.g. "2022-11-27T00:00:00"
            `total_votes` - total votes of the proposal
        limit : int
            number of votes, 0-1000
        order: str
            `by_creator` - order by proposal creator
            `by_start_date` - order by proposal start date
            `by_end_date` - order by proposal end date
            `by_total_votes` - order by proposal total votes
        direction : str, None
            `ascending` or `descending`
        status : str, None
            `all`, `inactive`, `active`, `expired`, or `votable`

        Returns
        -------
        list:
        """

        params = [[""], 1000, "by_creator", "ascending", "all"]

        if not isinstance(start, (str, int)):
            raise ValueError("`start` must be a voter acount name or proposal id.")
        params[0][0] = start
        params[1] = within_range(limit, 0, 1000)

        orders = ("by_creator", "by_start_date", "by_end_date", "by_total_votes")
        if order not in orders:
            raise ValueError("`order` is not supported.")
        params[2] = order

        if direction is not None:
            if direction not in ("ascending", "descending"):
                raise ValueError("`direction` is not supported.")
            params[3] = direction

        if status is not None:
            statuses = ("all", "inactive", "active", "expired", "votable")
            if status not in statuses:
                raise ValueError("`status` is not supported.")
            params[4] = status

        return self.api.list_proposals(params)

    def is_known_transaction(self, tid):
        """Only return true if the transaction has not expired
        or been invalidated. If this method is called
        with a VERY old transaction we will return false,
        use account_history_api.get_transaction.
        https://developers.hive.io/apidefinitions/#condenser_api.is_known_transaction

        Parameters
        ----------
        tid : str
            Any valid transaction id.

        Returns
        -------
        list:
        """

        valid_string(tid)
        return self.api.is_known_transaction([tid])

    def get_collateralized_conversion_requests(self, account):
        """Returns objects corresponding with `collateralized_convert` operations.
        https://developers.hive.io/apidefinitions/#condenser_api.get_collateralized_conversion_requests

        Parameters
        ----------
        account : str
            Any valid Hive account username.

        Returns
        -------
        list:
        """

        valid_string(account)
        return self.api.get_collateralized_conversion_requests([account])

    def find_recurrent_transfers(self, account):
        """Finds transfers of any liquid asset every fixed amount of time from one account to another.
        https://developers.hive.io/apidefinitions/#condenser_api.find_recurrent_transfers

        Parameters
        ----------
        account : str
            Any valid Hive account username.

        Returns
        -------
        list:
        """

        valid_string(account)
        return self.api.find_recurrent_transfers([account])

    def find_rc_accounts(self, accounts):
        """Find RC delegations from accounts.
        https://developers.hive.io/apidefinitions/#condenser_api.find_rc_accounts

        Parameters
        ----------
        accounts : list
            a list of any valid Hive account usernames

        Returns
        -------
        list:
        """

        params = [[accounts]]
        if not isinstance(accounts, str):
            if not isinstance(accounts, list):
                raise ValueError("`accounts` must be a list of strings.")
            if not all([0 for k in accounts if not isinstance(k, str)]):
                raise ValueError("`accounts` must be a list of strings.")
            params[0] = accounts
        return self.api.find_rc_accounts(params)

    def list_rc_accounts(self, start, limit):
        """List all RC delegations starting from the specified account.
        https://developers.hive.io/apidefinitions/#condenser_api.list_rc_accounts

        Parameters
        ----------
        start : str
            Hive account username, lower bound.
        limit : int
            Maximum number of results, from 1 up to ?.

        Returns
        -------
        list:
        """

        valid_string(start)
        within_range(limit, 1, 1000)
        return self.api.list_rc_accounts([start, limit])

    def list_rc_direct_delegations(self, delegator, delegatee, limit):
        """List all RC delegations starting from the specified account.
        https://developers.hive.io/apidefinitions/#condenser_api.list_rc_direct_delegations

        Parameters
        ----------
        account : str
            Hive account username, lower bound.
        limit : int
            Maximum number of results, from 1 up to ?.

        Returns
        -------
        list:
        """

        valid_string(delegator)
        valid_string(delegatee)
        within_range(limit, 1, 1000)
        return self.api.list_rc_direct_delegations([[delegator, delegatee], limit])



import os
import json
from arkivist import Arkivist
from sometime import Sometime
from nektar import Waggle
import nektar

def main():

    dt = Sometime().custom("%Y.%m.%d")

    counter = 0
    print("Initializing tests...")
    path = os.path.expanduser("~").replace("\\", "/")
    tests = Arkivist(path + "/nektar-tests.json", autosave=False)

    ##################################################
    # Waggle Class                                   #
    # Methods for blogging and engagement            #
    ##################################################

    counter += 1
    print(f"\nTEST #{counter}: Initialize Waggle class with dictionary of WIFs.")
    hive = Waggle(tests["username2"], wifs=tests["wifs2"])
    
    author = "trucklife-family"
    permlink = "seeing-the-impossible-made-possible"
    
    counter += 1
    print(f"\nTEST #{counter}: Voted")
    voted = hive.voted(author, permlink)
    print("Voted:", voted)
    
    counter += 1
    print(f"\nTEST #{counter}: Upvote")
    hive.vote(author, permlink, 1000, check=True)


if __name__ == "__main__":
    main()



# -*- coding: UTF-8 -*-
"""
Nektar for Python
~~~~~~~~~~~~~~~~

Hive API SDK built for Python.

"""

__version__ = "1.1.2"
from .appbase import AppBase
from .condenser import Condenser
from .nektar import Nektar, Waggle, Swarm

__all__ = ["appbase", "condenser", "nektar"]



# -*- coding: utf-8 -*-
"""
    nektar.transactions
    ~~~~~~~~~

    Sign serialized transactions with the private keys.

    :copyright: 2022 Rodney M. Maniego Jr.
    :license: MIT License
"""

import time
import array
import ecdsa
import struct
import hashlib
from binascii import hexlify, unhexlify

from .base58 import Base58


def sign_transaction(chain_id, serialized_transaction, wifs):
    """Sign serialized transaction with the private keys.

    :chain_id: dynamically accessed chain id
    :serialized_transaction: serialized transaction
    :wifs: list of private keys

    :param chain_id:
    :param serialized_transaction:
    :param wifs:

    """

    message = unhexlify(chain_id + serialized_transaction[:-2])
    digest = hashlib.sha256(message).digest()

    signatures = []
    for wif in wifs:
        i = 0
        p = Base58(wif).__bytes__()
        sk = ecdsa.SigningKey.from_string(p, curve=ecdsa.SECP256k1)
        while True:
            k = ecdsa.rfc6979.generate_k(
                sk.curve.generator.order(),
                sk.privkey.secret_multiplier,
                hashlib.sha256,
                hashlib.sha256(digest + struct.pack("d", time.time())).digest(),
            )

            sigder = sk.sign_digest(digest, sigencode=ecdsa.util.sigencode_der, k=k)

            r, s = ecdsa.util.sigdecode_der(sigder, sk.curve.generator.order())
            signature = ecdsa.util.sigencode_string(r, s, sk.curve.generator.order())

            sigder = array.array("B", sigder)
            lenR = sigder[3]
            lenS = sigder[5 + lenR]
            if lenR == 32 and lenS == 32:
                i = recover_pubkey_parameter(digest, signature, sk.get_verifying_key())
                i += 31  # compressed 4 + compact 27
                break

        sigstr = struct.pack("<B", i)
        sigstr += signature
        signatures.append(hexlify(sigstr).decode("ascii"))
    return signatures


def recover_pubkey_parameter(digest, signature, pubkey):
    """

    :param digest:
    :param signature:
    :param pubkey:

    """
    for i in range(0, 4):
        pk = recover_public_key(digest, signature, i)

        p = pk.pubkey.point
        order = pk.curve.generator.order()
        x = ecdsa.util.number_to_string(p.x(), order)
        compressed_pk = bytes(chr(2 + (p.y() & 1)), "ascii") + x
        if pubkey.to_string() in (pk.to_string(), compressed_pk):
            return i
    return None


def recover_public_key(digest, signature, i):
    """

    :param digest:
    :param signature:
    :param i:

    """
    curve = ecdsa.SECP256k1.curve
    G = ecdsa.SECP256k1.generator
    order = ecdsa.SECP256k1.order
    r, s = ecdsa.util.sigdecode_string(signature, order)
    x = r + (i // 2) * order
    alpha = ((x**3) + (curve.a() * x) + curve.b()) % curve.p()
    beta = ecdsa.numbertheory.square_root_mod_prime(alpha, curve.p())
    y = beta
    if (beta - (i % 2)) % 2 != 0:
        y = curve.p() - beta
    e = ecdsa.util.string_to_number(digest)
    R = ecdsa.ellipticcurve.Point(curve, x, y, order)
    Q = ecdsa.numbertheory.inverse_mod(r, order) * (s * R + (-e % order) * G)
    if not ecdsa.VerifyingKey.from_public_point(Q, curve=ecdsa.SECP256k1).verify_digest(
        signature, digest, sigdecode=ecdsa.util.sigdecode_string
    ):
        return None
    return ecdsa.VerifyingKey.from_public_point(Q, curve=ecdsa.SECP256k1)


# -*- coding: utf-8 -*-
"""
    nektar.constants
    ~~~~~~~~~

    This module contains all Hive blockchain and Nektar constants.
    Do not change to avoid unecessary breakage.
    
    Review the official Hive codebase and the API documentation
    before any modifications.

    :copyright: 2022 Rodney Maniego Jr.
    :license: MIT License
"""

import re

NEKTAR_VERSION = "1.1.0"

# PublicKey prefix
PREFIX = "STM"

"""
    AppBase APIs and methods
    The Condenser API is stable and can be used in production.
    Other AppBase APIs methods are still under development and subject to change.
"""
APPBASE_API = {
    "condenser_api": [
        "broadcast_block",
        "broadcast_transaction",
        "broadcast_transaction_synchronous",
        "find_proposals",
        "find_recurrent_transfers",
        "get_account_count",
        "get_account_history",
        "get_account_reputations",
        "get_accounts",
        "get_active_votes",
        "get_active_witnesses",
        "get_block",
        "get_block_header",
        "get_blog",
        "get_blog_authors",
        "get_blog_entries",
        "get_chain_properties",
        "get_collateralized_conversion_requests",
        "get_comment_discussions_by_payout",
        "get_config",
        "get_content",
        "get_content_replies",
        "get_conversion_requests",
        "get_current_median_history_price",
        "get_discussions_by_active",
        "get_discussions_by_author_before_date",
        "get_discussions_by_blog",
        "get_discussions_by_cashout",
        "get_discussions_by_children",
        "get_discussions_by_comments",
        "get_discussions_by_created",
        "get_discussions_by_feed",
        "get_discussions_by_hot",
        "get_discussions_by_promoted",
        "get_discussions_by_trending",
        "get_discussions_by_votes",
        "get_dynamic_global_properties",
        "get_escrow",
        "get_expiring_vesting_delegations",
        "get_feed",
        "get_feed_entries",
        "get_feed_history",
        "get_follow_count",
        "get_followers",
        "get_following",
        "get_hardfork_version",
        "get_key_references",
        "get_market_history",
        "get_market_history_buckets",
        "get_next_scheduled_hardfork",
        "get_open_orders",
        "get_ops_in_block",
        "get_order_book",
        "get_owner_history",
        "get_post_discussions_by_payout",
        "get_potential_signatures",
        "get_reblogged_by",
        "get_recent_trades",
        "get_recovery_request",
        "get_replies_by_last_update",
        "get_required_signatures",
        "get_reward_fund",
        "get_savings_withdraw_from",
        "get_savings_withdraw_to",
        "get_state",
        "get_tags_used_by_author",
        "get_ticker",
        "get_trade_history",
        "get_transaction",
        "get_transaction_hex",
        "get_trending_tags",
        "get_version",
        "get_vesting_delegations",
        "get_volume",
        "get_withdraw_routes",
        "get_witness_by_account",
        "get_witness_count",
        "get_witness_schedule",
        "get_witnesses",
        "get_witnesses_by_vote",
        "is_known_transaction",
        "list_proposal_votes",
        "list_proposals",
        "lookup_account_names",
        "lookup_accounts",
        "lookup_witness_accounts",
        "verify_authority",
        "find_rc_accounts",
        "list_rc_accounts",
        "list_rc_direct_delegations"
    ],
    "account_by_key_api": ["get_key_references"],
    "bridge": [
        "account_notifications",
        "does_user_follow_any_lists",
        "get_account_posts",
        "get_community",
        "get_community_context",
        "get_discussion",
        "get_follow_list",
        "get_payout_stats",
        "get_post",
        "get_post_header",
        "get_profile",
        "get_ranked_posts",
        "get_relationship_between_accounts",
        "list_all_subscriptions",
        "list_communities",
        "list_community_roles",
        "list_pop_communities",
        "list_subscribers",
    ],
    "account_history_api": [
        "enum_virtual_ops",
        "get_account_history",
        "get_ops_in_block",
        "get_transaction",
    ],
    "block_api": ["get_block", "get_block_header", "get_block_range"],
    "database_api": [
        "find_account_recovery_requests",
        "find_accounts",
        "find_change_recovery_account_requests",
        "find_collateralized_conversion_requests",
        "find_comments",
        "find_decline_voting_rights_requests",
        "find_escrows",
        "find_hbd_conversion_requests",
        "find_limit_orders",
        "find_owner_histories",
        "find_proposals",
        "find_recurrent_transfers",
        "find_savings_withdrawals",
        "find_vesting_delegation_expirations",
        "find_vesting_delegations",
        "find_votes",
        "find_withdraw_vesting_routes",
        "find_witnesses",
        "get_active_witnesses",
        "get_comment_pending_payouts",
        "get_config",
        "get_current_price_feed",
        "get_dynamic_global_properties",
        "get_feed_history",
        "get_hardfork_properties",
        "get_order_book",
        "get_potential_signatures",
        "get_required_signatures",
        "get_reward_funds",
        "get_transaction_hex",
        "get_version",
        "get_witness_schedule",
        "is_known_transaction",
        "list_account_recovery_requests",
        "list_accounts",
        "list_change_recovery_account_requests",
        "list_collateralized_conversion_requests",
        "list_comments",
        "list_decline_voting_rights_requests",
        "list_escrows",
        "list_hbd_conversion_requests",
        "list_limit_orders",
        "list_owner_histories",
        "list_proposal_votes",
        "list_proposals",
        "list_savings_withdrawals",
        "list_vesting_delegation_expirations",
        "list_vesting_delegations",
        "list_votes",
        "list_withdraw_vesting_routes",
        "list_witness_votes",
        "list_witnesses",
        "verify_authority",
        "verify_signatures",
    ],
    "debug_node_api": [
        "debug_generate_blocks",
        "debug_generate_blocks_until",
        "debug_get_hardfork_property_object",
        "debug_get_json_schema",
        "debug_get_witness_schedule",
        "debug_has_hardfork",
        "debug_pop_block",
        "debug_push_blocks",
        "debug_set_hardfork",
    ],
    "follow_api": [
        "get_account_reputations",
        "get_blog",
        "get_blog_authors",
        "get_blog_entries",
        "get_feed",
        "get_feed_entries",
        "get_follow_count",
        "get_followers",
        "get_following",
        "get_reblogged_by",
    ],
    "market_history_api": [
        "get_market_history",
        "get_market_history_buckets",
        "get_order_book",
        "get_recent_trades",
        "get_ticker",
        "get_trade_history",
        "get_volume",
    ],
    "network_broadcast_api": ["broadcast_block", "broadcast_transaction"],
    "rc_api": ["find_rc_accounts", "get_resource_params", "get_resource_pool"],
    "reputation_api": ["get_account_reputations"],
}

DISCUSSIONS_BY = ("active", "blog", "cashout", "children", "created", "hot", "payout", "promoted", "trending", "votes")

"""
    Hive Blockchain Operations
    Indices reflect its equivalent integer value (w/ 128-bit bitmasking)
    WARNING: Do NOT change order of operations!
"""
BLOCKCHAIN_OPERATIONS = [
    "vote",
    "comment",
    "transfer",
    "transfer_to_vesting",
    "withdraw_vesting",
    "limit_order_create",
    "limit_order_cancel",
    "feed_publish",
    "convert",
    "account_create",
    "account_update",
    "witness_update",
    "account_witness_vote",
    "account_witness_proxy",
    "pow",
    "custom",
    "report_over_production",
    "delete_comment",
    "custom_json",
    "comment_options",
    "set_withdraw_vesting_route",
    "limit_order_create2",
    "claim_account",
    "create_claimed_account",
    "request_account_recovery",
    "recover_account",
    "change_recovery_account",
    "escrow_transfer",
    "escrow_dispute",
    "escrow_release",
    "pow2",
    "escrow_approve",
    "transfer_to_savings",
    "transfer_from_savings",
    "cancel_transfer_from_savings",
    "custom_binary",
    "decline_voting_rights",
    "reset_account",
    "set_reset_account",
    "claim_reward_balance",
    "delegate_vesting_shares",
    "account_create_with_delegation",
    "witness_set_properties",
    "account_update2",
    "create_proposal",
    "update_proposal_votes",
    "remove_proposal",
    "fill_convert_request",
    "author_reward",
    "curation_reward",
    "comment_reward",
    "liquidity_reward",
    "producer_reward",
    "interest",
    "fill_vesting_withdraw",
    "fill_order",
    "shutdown_witness",
    "fill_transfer_from_savings",
    "hardfork",
    "comment_payout_update",
    "return_vesting_delegation",
    "comment_benefactor_reward",
    "return_vesting_delegation",
    "comment_benefactor_reward",
    "producer_reward",
    "clear_null_account_balance",
    "proposal_pay",
    "sps_fund",
    "hardfork_hive",
    "hardfork_hive_restore",
]

# HIVE PRIVATE KEY ROLES
ROLES = {
    "all": ["owner", "active", "posting", "memo"],
    "comment": ["posting", "active", "owner"],
    "follow": ["posting", "active", "owner"],
    "vote": ["posting", "active", "owner"],
    "transfer_to_vesting": ["active", "owner"],
    "transfer_to_savings": ["active", "owner"],
    "transfer": ["active", "owner"],
    "custom_json": ["active"],
    "owner": ["owner"],
    "active": ["active"],
    "posting": ["posting"],
    "memo": ["memo"],
}

# HIVE ASSETS
ASSETS = {
    "HBD": {"nai": "@@000000013", "precision": 3},
    "HIVE": {"nai": "@@000000021", "precision": 3},
    "VESTS": {"nai": "@@000000037", "precision": 6},
}

# Blockchain Transaction Expiration Format in UTC
DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%S%Z"

"""
    Hive Nodes
    Nodes availability and reliability may change.
    Custom nodes may be specified to override these options.
"""
NODES = [
    "api.hive.blog",
    "api.openhive.network",
    "anyx.io",
    "hived.privex.io",
    "rpc.ausbit.dev",
    "techcoderx.com",
    "rpc.ecency.com",
    "hive.roelandp.nl",
    "hived.emre.sh",
    "api.deathwing.me",
    "api.c0ff33a.uk",
    "hive-api.arcange.eu",
    # "api.pharesim.me",
]

        
RE_PROTOCOL = re.compile(r"http[s]{0,1}\:[\/]{2}")
RE_URL_PATH = re.compile(r"[\/][\w\W]+")
RE_USERNAME = re.compile(r"[a-z][\w\.\-]{2,15}")
RE_SNAKE_CASE = re.compile(r"[^\w\-]+")
RE_PERMLINK = re.compile(r"[\w\-\%]{0,255}")
RE_COMMUNITY = re.compile(r"\bhive-[\d]{1,}")
RE_DATETIME = re.compile(r"\d{4}\-\d{2}\-\d{2}T\d{2}:\d{2}:\d{2}")
RE_NEWLINES = re.compile(r"[\r\n]")
RE_WORDS = re.compile(r"[^\w\ ]")
RE_IMAGES = re.compile(r"!\[[^\]]*\]\([^\)]+\)")
RE_NUMERIC = re.compile(r"\d+")


import os
import json
from arkivist import Arkivist
from sometime import Sometime
from nektar import Nektar

def main():

    dt = Sometime().custom("%Y.%m.%d")

    counter = 0
    print("Initializing tests...")
    path = os.path.expanduser("~").replace("\\", "/")
    tests = Arkivist(path + "/nektar-tests.json", autosave=False)

    ##################################################
    # Nektar Class / Base Class                      #
    ##################################################
    
    # custom_nodes = ["testnet.openhive.network"]
    # TESTNET = "18dcf0a285365fc58b71f18b3d3fec954aa0c141c44e4e5cb4cf777b9eab274e"

    print(f"\nTEST: Initialize Nektar class with dictionary of WIFs.")
    hive = Nektar(tests["username0"], wifs=tests["wifs0"])

    print(f"\nTEST: Initialize Nektar class with app and version")
    hive = Nektar(tests["username0"], wifs=tests["wifs0"], app="nektar.app", version=dt)

    print(f"\nTEST: Get the list of blockchain constants")
    # Hive Developer Portal > Understanding Configuration Values
    # https://developers.hive.io/tutorials-recipes/understanding-configuration-values.html
    data = hive.get_config()
    print(json.dumps(data, indent=2))

    print(f"\nTEST: Get the list of blockchain constants")
    data = hive.get_config(field="HIVE_CHAIN_ID", fallback="bee*")
    print("HIVE_CHAIN_ID: " + str(data))

    print(
        f"\nTEST: Get the initialized account's recent resource credits."
    )
    data = hive.resource_credits()
    print(data)

    print(f"\nTEST: Get another account's recent resource credits.")
    data = hive.resource_credits(tests["username0"])
    print(data)

    print(
        f"\nTEST: Get the initialized account's remaining mana in percentage."
    )
    percentage = hive.manabar()
    print("Current Mana: " + str(int(percentage)) + "%")

    print(
        f"\nTEST: Get another account's remaining mana in percentage."
    )
    percentage = hive.manabar()
    print("Current Mana: " + str(int(percentage)) + "%")

    print(f"\nTEST: Transfer HBD to another account with a message.")
    receiver = tests["username1"]
    amount = 0.001
    asset = "HBD"
    message = "Thanks for supporting us! - test"
    hive.memo(receiver, amount, asset, message, mock=True)

    print(f"\nTEST: Transfer HIVE to another account with a message.")
    receiver = tests["username1"]
    amount = 0.001
    asset = "HIVE"
    message = "Thanks for supporting us! - test"
    result = hive.memo(receiver, amount, asset, message, mock=True)

    print(f"\nTEST: Transfer HBD to savings with a message.")
    receiver = tests["username0"]
    amount = 0.001
    asset = "HBD"
    message = "Thanks for supporting us! - test"
    hive.transfer_to_savings(receiver, amount, asset, message, mock=True)

    print(f"\nTEST: Transfer HIVE to savings with a message.")
    receiver = tests["username0"]
    amount = 0.001
    asset = "HIVE"
    message = "Thanks for supporting us! - test"
    hive.transfer_to_savings(receiver, amount, asset, message, mock=True)

    print(f"\nTEST: Transfer to vesting.")
    receiver = tests["username0"]
    amount = 0.001
    hive.transfer_to_vesting(receiver, amount, mock=True)

    print(f"\nTEST: Broadcast a custom JSON.")
    auths = [tests["username0"]]
    hive.custom_json(
        id_="nektar-tests",
        jdata={ "test": "nektar" },
        required_auths=auths, mock=True)

    print(f"\nTEST: Broadcast a custom JSON.")
    auths = [tests["username0"]]
    hive.custom_json(
        id_="nektar-tests",
        jdata={ "test": "nektar" },
        required_posting_auths=auths, mock=True)

if __name__ == "__main__":
    main()



import os
import json
from arkivist import Arkivist
from sometime import Sometime
from nektar import Condenser

def main():

    counter = 0
    print("Initializing tests...")

    ##################################################
    # Condenser Class                                #
    # CondenserAPI methods                           #
    ##################################################

    print(f"\nTEST: Initialize Condenser class.")
    hive = Condenser()

    start = 1
    limit = 1
    tag = "hive"
    account = "hiveio"
    author = "hivehealth"
    stimestamp = "2022-01-01T00:00:00"
    etimestamp = "2022-01-02T00:00:00"
    permlink = "the-updated-easy-onboarding-manual"
    tid = "6fde0190a97835ea6d9e651293e90c89911f933c"
    tx = {
        "ref_block_num": 56236,
        "ref_block_prefix": 2845561047,
        "expiration": "2022-11-30T11:55:00",
        "operations": [
            ["custom_json", {"required_auths": [account], "required_posting_auths": [], "id": "contract_id", "json": "{}"}]],
        "extensions": []
    }
    keys = ["STM6vJmrwaX5TjgTS9dPH8KsArso5m91fVodJvv91j7G765wqcNM9",
            "STM6vJmrwaX5TjgTS9dPH8KsArso5m91fVodJvv91j7G765wqcNM9"]
    
    """
    print(f"\nTEST: get_account_count")
    hive.get_account_count()
    
    print(f"\nTEST: get_account_history, operation = all")
    hive.get_account_history(account, start, limit)

    print(f"\nTEST: get_account_history, operation = 0")
    hive.get_account_history(account, start, limit, low=0)

    print(f"\nTEST: get_account_reputations")
    hive.get_account_reputations(start="a", limit=5)
    
    print(f"\nTEST: get_account_reputations")
    hive.get_account_reputations("ba", 10)
    
    print(f"\nTEST: get_accounts")
    accounts = ["gtg"]
    delayed_votes_active = True
    hive.get_accounts(accounts, delayed_votes_active)
    
    print(f"\nTEST: get_accounts")
    accounts = ["gtg", "abc"]
    hive.get_accounts(accounts, False)
    
    print(f"\nTEST: get_active_votes")
    hive.get_active_votes(author, permlink)
    
    print(f"\nTEST: get_active_witnesses")
    hive.get_active_witnesses()
    
    print(f"\nTEST: get_block")
    hive.get_block(1)
    
    print(f"\nTEST: get_block_header")
    hive.get_block_header(1)
    
    print(f"\nTEST: get_blog")
    start_entry_id = 10
    hive.get_blog(account, start_entry_id, limit)
    
    ## failing
    print(f"\nTEST: get_blog_authors")
    # hive.get_blog_authors(account)
    
    print(f"\nTEST: get_blog_entries")
    hive.get_blog_entries(account, 1, 1)
    
    print(f"\nTEST: get_chain_properties")
    hive.get_chain_properties()
    
    print(f"\nTEST: get_comment_discussions_by_payout")
    tag = "ctp"
    hive.get_comment_discussions_by_payout(tag, limit=10, truncate=1)
    
    print(f"\nTEST: get_config")
    hive.get_config()
    
    print(f"\nTEST: get_content")
    hive.get_content(author, permlink)
    
    print(f"\nTEST: get_content_replies")
    hive.get_content_replies(author, permlink)
    
    print(f"\nTEST: get_conversion_requests")
    hive.get_conversion_requests(account)
    
    print(f"\nTEST: get_current_median_history_price")
    hive.get_current_median_history_price()
    
    # Assert Exception:false: Supported by hivemind
    print(f"\nTEST: get_discussions_by_active")
    # hive.get_discussions_by_active(tag, limit=2, truncate=0)
    
    print(f"\nTEST: get_discussions_by_author_before_date")
    date = "2022-12-30T00:00:00"
    hive.get_discussions_by_author_before_date(author, "", date, limit=1)

    print(f"\nTEST: get_discussions_by_blog")
    hive.get_discussions_by_blog(tag, limit=2, truncate=0)
    
    # Assert Exception:false: Supported by hivemind
    print(f"\nTEST: get_discussions_by_cashout")
    # hive.get_discussions_by_cashout(tag, limit=2, truncate=0)
    
    # Assert Exception:false: Supported by hivemind
    print(f"\nTEST: get_discussions_by_children")
    # hive.get_discussions_by_children(tag, limit=2, truncate=0)

    print(f"\nTEST: get_discussions_by_comments")
    author = "hive"
    permlink = ""
    hive.get_discussions_by_comments(author, permlink, limit=1)
    
    # Assert Exception:false: Supported by hivemind
    print(f"\nTEST: get_discussions_by_created")
    # hive.get_discussions_by_created(tag, limit=2, truncate=0)
    
    print(f"\nTEST: get_discussions_by_feed")
    hive.get_discussions_by_feed(tag, author="", permlink="", limit=1)
    
    print(f"\nTEST: get_discussions_by_hot")
    hive.get_discussions_by_hot(tag, limit=2, truncate=0)
    
    print(f"\nTEST: get_discussions_by_promoted")
    hive.get_discussions_by_promoted(tag, limit=2, truncate=0)
    
    print(f"\nTEST: get_discussions")
    by = "votes"
    hive.get_discussions(by, tag, limit=2, truncate=0)
    
    print(f"\nTEST: get_escrow")
    hive.get_escrow(account="hiveio", eid=0)
    
    print(f"\nTEST: get_expiring_vesting_delegations")
    account = "hiveio"
    after = "2018-01-01T00:00:00"
    hive.get_expiring_vesting_delegations(account, after)
    
    # Assert Exception:false: Supported by hivemind
    print(f"\nTEST: get_feed")
    account = "hiveio"
    # hive.get_feed(account, eid=0, limit=1)
    
    # Assert Exception:false: Supported by hivemind
    print(f"\nTEST: get_feed_entries")
    account = "hiveio"
    # hive.get_feed_entries(account, eid=0, limit=1)
    
    print(f"\nTEST: get_follow_count")
    hive.get_follow_count(account)
    
    print(f"\nTEST: get_followers")
    hive.get_followers(account, start="", ftype="blog", limit=5)
    
    print(f"\nTEST: get_following")
    hive.get_following(account, start="", ftype="blog", limit=5)
    
    print(f"\nTEST: get_hardfork_version")
    hive.get_hardfork_version()
    
    print(f"\nTEST: get_key_references")
    hive.get_key_references(keys)
    
    # TODO: re-validate
    print(f"\nTEST: get_market_history")
    hive.get_market_history(seconds=3600, start=stimestamp, end=etimestamp)
    
    print(f"\nTEST: get_market_history_buckets")
    hive.get_market_history_buckets()
    
    print(f"\nTEST: get_next_scheduled_hardfork")
    hive.get_next_scheduled_hardfork()
    
    print(f"\nTEST: get_open_orders")
    hive.get_open_orders(account)
    
    print(f"\nTEST: get_ops_in_block")
    hive.get_ops_in_block(number=5443322, virtual=True)
    
    print(f"\nTEST: get_order_book")
    hive.get_order_book(limit=5)
    
    print(f"\nTEST: get_owner_history")
    hive.get_owner_history(account)
    
    print(f"\nTEST: get_potential_signatures")
    hive.get_potential_signatures(tx)
    
    print(f"\nTEST: get_reblogged_by")
    hive.get_reblogged_by(author, permlink)
    
    print(f"\nTEST: get_recent_trades")
    hive.get_recent_trades(limit=5)
    
    print(f"\nTEST: get_recovery_request")
    hive.get_recovery_request(account)
    
    print(f"\nTEST: get_replies_by_last_update")
    hive.get_replies_by_last_update(author="hive", permlink="", limit=1)
    
    print(f"\nTEST: get_required_signatures")
    hive.get_required_signatures(transaction=tx, keys=[])
    
    print(f"\nTEST: get_reward_fund")
    hive.get_reward_fund(action="post")
    
    print(f"\nTEST: get_savings_withdraw_from")
    hive.get_savings_withdraw_from(account)
    
    print(f"\nTEST: get_savings_withdraw_to")
    hive.get_savings_withdraw_to(account)
    
    # Assert Exception:false: Supported by hivemind
    print(f"\nTEST: get_tags_used_by_author")
    # hive.get_tags_used_by_author(account)
    
    print(f"\nTEST: get_ticker")
    hive.get_ticker()
    
    print(f"\nTEST: get_trade_history")
    hive.get_trade_history(start=stimestamp, end=etimestamp, limit=10)
    
    print(f"\nTEST: get_transaction")
    hive.get_transaction(tid)
    
    print(f"\nTEST: get_transaction_hex")
    hive.get_transaction_hex(transaction=tx)
    
    print(f"\nTEST: get_trending_tags")
    hive.get_trending_tags(start="", limit=10)
    
    print(f"\nTEST: get_version")
    hive.get_version()
    
    print(f"\nTEST: get_vesting_delegations")
    hive.get_vesting_delegations(account, start="", limit=5)
    
    print(f"\nTEST: get_volume")
    hive.get_volume()
    
    print(f"\nTEST: get_withdraw_routes")
    hive.get_withdraw_routes(account, route="all")
    
    print(f"\nTEST: get_witness_by_account")
    hive.get_witness_by_account(account)
    
    print(f"\nTEST: get_witness_count")
    hive.get_witness_count()
    
    print(f"\nTEST: get_witness_count")
    hive.get_witness_count()
    
    print(f"\nTEST: get_witness_schedule")
    hive.get_witness_schedule()
    
    print(f"\nTEST: get_witnesses")
    hive.get_witnesses(indices=[0,19,20])
    
    print(f"\nTEST: get_witnesses_by_vote")
    hive.get_witnesses_by_vote(start="a", limit=5)
    
    print(f"\nTEST: lookup_account_names")
    hive.lookup_account_names(accounts=[account], delayed_votes_active=False)
    
    print(f"\nTEST: lookup_accounts")
    hive.lookup_accounts(start="a", limit=5)
    
    print(f"\nTEST: find_proposals")
    hive.find_proposals(pid=0)
    
    print(f"\nTEST: list_proposal_votes, by proposal voter")
    order = "by_proposal_voter"
    direction = "ascending"
    hive.list_proposal_votes(start, limit, order, direction, status="all")
    
    # Assert Exception:0 < args.limit
    # && args.limit <= DATABASE_API_SINGLE_QUERY_LIMIT:
    # limit not set or too big
    print(f"\nTEST: list_proposal_votes, by voter proposal")
    start = "gtg"
    order = "by_voter_proposal"
    direction = "descending"
    status = "active"
    hive.list_proposal_votes(start, limit, order, direction, status)
    
    # Assert Exception:0 < args.limit
    # && args.limit <= DATABASE_API_SINGLE_QUERY_LIMIT:
    # limit not set or too big
    print(f"\nTEST: list_proposals, by creator")
    start = "gtg"
    order = "by_creator"
    direction = "ascending"
    status = "active"
    hive.list_proposals(start, limit, order, direction, status)
    
    # Assert Exception:0 < args.limit
    # && args.limit <= DATABASE_API_SINGLE_QUERY_LIMIT:
    # limit not set or too big
    print(f"\nTEST: list_proposals, by start date")
    start = "2022-01-01T00:00:00"
    limit = 5
    order = "by_start_date"
    direction = "ascending"
    status = "active"
    hive.list_proposals(start, limit, order, direction, status)
    
    # Assert Exception:0 < args.limit
    # && args.limit <= DATABASE_API_SINGLE_QUERY_LIMIT:
    # limit not set or too big
    print(f"\nTEST: list_proposals, by end date")
    start = "2022-12-30T00:00:00"
    limit = 5
    order = "by_end_date"
    direction = "ascending"
    status = "active"
    hive.list_proposals(start, limit, order, direction, status)
    
    # Assert Exception:0 < args.limit
    # && args.limit <= DATABASE_API_SINGLE_QUERY_LIMIT:
    # limit not set or too big
    print(f"\nTEST: list_proposals, by total votes")
    start = 0
    limit = 5
    order = "by_total_votes"
    direction = "ascending"
    status = "active"
    hive.list_proposals(start, limit, order, direction, status)
    
    print(f"\nTEST: is_known_transaction")
    hive.is_known_transaction(tid)
    
    print(f"\nTEST: get_collateralized_conversion_requests")
    hive.get_collateralized_conversion_requests(account)
    
    print(f"\nTEST: find_recurrent_transfers")
    hive.find_recurrent_transfers(account)
    
    print(f"\nTEST: find_rc_accounts")
    hive.find_rc_accounts(accounts=account)
    
    print(f"\nTEST: list_rc_accounts")
    hive.list_rc_accounts(account, limit=5)
    
    print(f"\nTEST: list_rc_direct_delegations")
    hive.list_rc_direct_delegations(delegator="oniemaniego", delegatee="hivehealth", limit=5)
    """

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
    nektar.appbase
    ~~~~~~~~~

    Interact with the Hive APIs.

    :copyright: 2022 Rodney Maniego Jr.
    :license: MIT License
"""

import json
import hashlib
import requests
import warnings
from re import sub
from binascii import hexlify, unhexlify
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from .mock import mock_server
from .transactions import sign_transaction
from .constants import (
    NEKTAR_VERSION,
    NODES,
    APPBASE_API,
    BLOCKCHAIN_OPERATIONS,
    ROLES,
    RE_PROTOCOL,
    RE_URL_PATH,
)


class AppBase:
    """Base SDK to communicate with the Hive APIs.
    ~~~~~~~~~

    :param nodes: a list of valid custom Hive nodes (Default value = None)
    :param api: a valid AppBase API (Default value = None)
    :param timeout: seconds before the request is dropped (Default value = 10)
    :param retries: a dictionary of roles and their equivalent WIFs (Default value = 3)
    :param warning: display warning messages (Default value = False)

    """

    def __init__(
        self, nodes=None, api=None, chain_id=None, timeout=10, retries=3, warning=False
    ):

        # set default to condenser api
        self._appbase_api = self.api(api)
        self.method = None
        self.rid = 0

        if not isinstance(warning, bool):
            raise TypeError("Warning must be `True` or `False` only.")
        if not warning:
            warnings.filterwarnings(
                "ignore",
                ".*unavailable.*",
            )

        # set session retries
        self.retries = 3
        self.set_retries(retries)

        # set session timeout
        self.timeout = 10
        self.set_timeout(timeout)

        # initialize session
        self.session = requests.Session()
        self.session.mount("http://", HTTPAdapter(max_retries=Retry(
            total=self.retries, backoff_factor=0.5, status_forcelist=[502, 503, 504]
        )))
        self.session.headers.update({
            "User-Agent": f"Nektar v{NEKTAR_VERSION}",
            "content-type": "application/json; charset=utf-8",
        })

        self.nodes = NODES
        self.custom_nodes(nodes)

        self.wifs = {}
        self.chain_id = chain_id
        self._transaction_id = None
        self.signed_transaction = None

    def custom_nodes(self, nodes):
        """Replace known nodes with custom nodes.

        :param nodes: a list of valid Hive nodes

        """
        if nodes is None:
            return
        if isinstance(nodes, str):
            nodes = [nodes]
        self.node = []

        for node in list(nodes):
            node = RE_PROTOCOL.sub("", node)
            node = RE_URL_PATH.sub("", node)
            if not len(node):
                raise TypeError("Invalid node format.")
            self.nodes.append(node)

    def append_wif(self, wif, role=None):
        """Append WIF and the associated role of it for signing transactions.

        If `wif` is empty, role is expected to be a dictionary containing a valid role-wif pairs.

        :param wif: a valid private key (Default value = None)
        :param role: role of private key being appended, must be `owner`, `active`, `posting`, or `memo` only.

        """
        if isinstance(wif, dict):
            for r, w in wif.items():
                self.append_wif(w, r)
            return
        if not isinstance(wif, str) and not isinstance(role, str):
            raise TypeError("Role and WIF must be a valid string.")
        if role not in ROLES["all"]:
            raise ValueError(
                "Role must be `owner`, `active`, `posting`, or `memo` only."
            )
        if wif not in list(self.wifs.values()):
            self.wifs[role] = wif

    def set_retries(self, retries):
        """

        :param retries: the number of retries when reconnect with the URLs

        """
        if 1 <= int(retries) <= 10:
            self.retries = retries

    def set_timeout(self, timeout):
        """

        :param timeout: seconds before the request is dropped

        """
        if 5 <= int(timeout) <= 120:
            self.timeout = timeout

    ## AppBase APIs
    def api(self, name):
        """Set to a valid API.

        :param name: a valid AppBase API

        """
        if name is None:
            self._appbase_api = "condenser_api"
            return self
        if isinstance(name, str):
            name = name.replace("_api", "")
            if name not in ("bridge",):
                name += "_api"
        if name not in APPBASE_API:
            raise ValueError(name + " is unsupported.")
        self._appbase_api = name
        return self

    def condenser(self):
        """Set the active API to `condenser_api`."""
        self._appbase_api = "condenser_api"
        return self

    def account_by_key(self):
        """Set the active API to `account_by_key_api`."""
        self._appbase_api = "account_by_key_api"
        return self

    def bridge(self):
        """Set the active API to `bridge`."""
        self._appbase_api = "bridge"
        return self

    def account_history(self):
        """Set the active API to `account_history_api`."""
        self._appbase_api = "account_history_api"
        return self

    def block(self):
        """Set the active API to `block_api`."""
        self._appbase_api = "block_api"
        return self

    def database(self):
        """Set the active API to `database_api`."""
        self._appbase_api = "database_api"
        return self

    def mock_node(self):
        """Set the active API to `mock_node_api`."""
        self._appbase_api = "mock_node_api"
        return self

    def follow(self):
        """Set the active API to `follow_api`."""
        self._appbase_api = "follow_api"
        return self

    def market_history(self):
        """Set the active API to `market_history_api`."""
        self._appbase_api = "market_history_api"
        return self

    def network_broadcast(self):
        """Set the active API to `network_broadcast_api`."""
        self._appbase_api = "network_broadcast_api"
        return self

    def rc(self):
        """Set the active API to `rc_api`."""
        self._appbase_api = "rc_api"
        return self

    def reputation(self):
        """Set the active API to `reputation_api`."""
        self._appbase_api = "reputation_api"
        return self

    ## API methods
    def __getattr__(self, method):
        def callable(*args, **kwargs):
            """Dynamically send an API request using a method call.

            :param *args:
            :param **kwargs:

            """
            return self._dynamic_api_call(method, *args, **kwargs)

        return callable

    def _dynamic_api_call(self, *args, **kwargs):
        """Dynamically send an API request using a method call.

        :param *args:
        :param **kwargs:

        """
        method = args[0]
        if method not in APPBASE_API[self._appbase_api]:
            raise ValueError(f"{name} is unsupported.")
        method = f"{self._appbase_api}.{method}"

        params = []
        if len(args) > 1:
            params = args[1]
        else:
            if self._appbase_api == "condenser_api":
                params = {}

        # raise exception on error or not
        strict = True
        if "strict" in kwargs:
            strict = kwargs["strict"]

        # use mock server or not
        mock = False
        if "mock" in kwargs:
            mock = kwargs["mock"]

        ## do not change sorting order !!
        broadcast_methods = [
            "condenser_api.verify_authority",
            "condenser_api.broadcast_transaction",
            "condenser_api.broadcast_transaction_synchronous",
            "database_api.get_potential_signatures",
            "database_api.get_required_signatures",
            "database_api.get_transaction_hex",
            "database_api.verify_authority",
            "network_broadcast_api.broadcast_transaction",
        ]

        if method in broadcast_methods:
            params = [params]
            if method in broadcast_methods[3:]:
                params = {"trx": params[0]}

        if method in broadcast_methods[1:]:
            return self.broadcast(method, params, strict=strict, mock=mock)
        return self.request(method, params, strict=strict, mock=mock)

    def request(self, method, params, strict=True, mock=False):
        """Send predefined params as JSON-RPC request.

        :param method:
        :param params:

        """
        self.rid += 1
        payload = _format_payload(method, params, self.rid)
        return self._send_request(payload, strict=strict, mock=mock)

    def broadcast(self, method, transaction, strict=True, mock=False):
        """Broadcast a transaction to the blockchain.

        :param method:
        :param transaction:
        :param strict:  (Default value = True)
        :param mock:  (Default value = False)

        """

        if not isinstance(self.chain_id, str):
            self.chain_id = self.api("database").get_version({})["chain_id"]

        ## check if operations are valid
        operation = ""
        for op in transaction["operations"]:
            if op[0] not in BLOCKCHAIN_OPERATIONS:
                raise ValueError(op[0] + " is unsupported")
            operation = op[0]
            # avoid unnecessary signatures
            if operation == "custom_json":
                if not len(op[1]["required_auths"]):
                    operation = "posting"

        # signatures = []
        # if "signatures" in transaction:
        #     signatures = transaction["signatures"]
        #     transaction.pop("transaction", None)

        ## serialize transaction and sign with private keys
        serialized_transaction = self._serialize(transaction)

        ## generate transaction id
        hashed = hashlib.sha256(unhexlify(serialized_transaction)).digest()
        # self._transaction_id = hexlify(hashed[:20]).decode("ascii")

        ## update transaction signature
        self.signed_transaction = transaction
        wifs = _get_necessary_wifs(self.wifs, operation)
        self.signed_transaction["signatures"] = sign_transaction(
            self.chain_id, serialized_transaction, wifs
        )

        if strict:
            verified = self.api("condenser").verify_authority(self.signed_transaction)
            if not verified:
                raise ValueError("Transaction does not contain required signatures.")

        self.rid += 1
        payload = _format_payload(method, [self.signed_transaction], self.rid)
        return self._send_request(payload, strict, mock)

    def _serialize(self, transaction):
        """Get a hexdump of the serialized binary form of a transaction.

        :param transaction:

        """
        return self.api("condenser").get_transaction_hex([transaction])

    def _send_request(self, payload, strict=True, mock=False):
        """Send an API request with a valid payload, as defined by the Hive API documentation.

        :param payload: a formatted and valid payload.
        :param strict: flag to cause exception upon encountering an error (Default value = True)

        """
        if mock:
            return mock_server(payload)
        
        # send request to next node when failing
        data = {}
        for node in self.nodes:
            try:
                response = self.session.post(
                    f"https://{node}", json=payload, timeout=self.timeout
                )
                response.raise_for_status()
                data = json.loads(response.content.decode("utf-8"))
                break
            except:
                warnings.warn(
                    f"Node '{node}' is unavailable, retrying with the next node."
                )
        if strict and ("error" in data):
            raise SystemError(data["error"].get("message"))
        return data.get("result", {})


#########################
# utils                 #
#########################


def _get_necessary_wifs(wifs, operation):
    """

    :param wifs: a list of WIFs
    :param operation: a valid operation to be performed in the Hive blockchain.

    """
    for role in ROLES[operation]:
        if role in wifs:
            return [wifs[role]]
    return []


def _format_payload(method, params, rid):
    """Format API method parameters to conform to the Hive API documentation.

    :param method: the API method
    :param params: the required parameters to successfully perform the operation or request
    :param rid: the unique RPC id used for mockging purposes

    """
    return {"jsonrpc": "2.0", "method": method, "params": params, "id": rid}


def _make_expiration(secs=30):
    """Create an expiration time based on the current UTC datetime.

    :param expire: transaction expiration in seconds (Default value = 30)

    """
    timestamp = time.time() + int(secs)
    return datetime.utcfromtimestamp(timestamp).strftime(BLOCKCHAIN_DT_FORMAT)


# -*- coding: utf-8 -*-
"""
    nektar.mock
    ~~~~~~~~~

    :copyright: 2022 Rodney Maniego Jr.
    :license: MIT License
"""


def mock_server(payload):
    """Returns a mock result based on the payload.

    Parameters
    ----------
    payload :
        a valid transaction payload
    """
    if not isinstance(payload, dict):
        raise TypeError("`payload` must be a valid JSON RPC v2.0 transaction.")
    method = payload.get("method")

    if method is None:
        raise ValueError("`method` is missing on payload.")

    if method in (
        "condenser_api.get_account_history",
        "condenser_api.get_account_reputations",
        "condenser_api.get_active_witnesses",
        "condenser_api.get_blog",
        "condenser_api.get_blog_entries",
        "condenser_api.get_comment_discussions_by_payout",
        "condenser_api.get_conversion_requests",
        "condenser_api.get_discussions_by_active",
        "condenser_api.get_discussions_by_author_before_date",
        "condenser_api.get_discussions_by_blog",
        "condenser_api.get_discussions_by_cashout",
        "condenser_api.get_discussions_by_children",
        "condenser_api.get_discussions_by_comments",
        "condenser_api.get_discussions_by_created",
        "condenser_api.get_discussions_by_feed",
        "condenser_api.get_discussions_by_hot",
        "condenser_api.get_discussions_by_promoted",
        "condenser_api.get_discussions_by_trending",
        "condenser_api.get_discussions_by_votes",
        "condenser_api.get_expiring_vesting_delegations",
        "condenser_api.get_feed",
        "condenser_api.get_feed_entries",
        "condenser_api.get_followers",
        "condenser_api.get_following",
        "condenser_api.get_key_references",
        "condenser_api.get_market_history",
        "condenser_api.get_market_history_buckets",
        "condenser_api.get_open_orders",
        "condenser_api.get_ops_in_block",
        "condenser_api.get_owner_history",
        "condenser_api.get_post_discussions_by_payout",
        "condenser_api.get_potential_signatures",
        "condenser_api.get_reblogged_by",
        "condenser_api.get_replies_by_last_update",
        "condenser_api.get_required_signatures",
        "condenser_api.get_savings_withdraw_from",
        "condenser_api.get_savings_withdraw_to",
        "condenser_api.get_trade_history",
        "condenser_api.get_trending_tags",
        "condenser_api.get_vesting_delegations",
        "condenser_api.get_withdraw_routes",
        "condenser_api.get_witnesses",
        "condenser_api.get_witnesses_by_vote",
        "condenser_api.lookup_accounts",
        "condenser_api.lookup_witness_accounts",
        "condenser_api.get_account_references",
        "condenser_api.find_proposals",
        "condenser_api.list_proposal_votes",
        "condenser_api.list_proposals",
        "condenser_api.get_collateralized_conversion_requests",
        "condenser_api.find_recurrent_transfers",
        "condenser_api.find_rc_accounts",
        "condenser_api.list_rc_accounts",
        "condenser_api.list_rc_direct_delegations",
        "bridge.get_ranked_posts",
        "bridge.account_notifications",
        "bridge.get_account_posts",
        "bridge.get_follow_list",
        "bridge.list_communities",
        "bridge.list_pop_communities",
        "bridge.list_subscribers",
        "bridge.list_community_roles",
        "bridge.list_all_subscriptions",
        "jsonrpc.get_methods",
        "wallet_bridge_api.find_rc_accounts",
        "wallet_bridge_api.find_recurrent_transfers",
        "wallet_bridge_api.get_account_history",
        "wallet_bridge_api.get_accounts",
        "wallet_bridge_api.get_collateralized_conversion_requests",
        "wallet_bridge_api.get_conversion_requests",
        "wallet_bridge_api.get_open_orders",
        "wallet_bridge_api.get_withdraw_routes",
        "wallet_bridge_api.list_accounts",
        "wallet_bridge_api.list_my_accounts",
        "wallet_bridge_api.list_rc_accounts",
        "wallet_bridge_api.list_rc_direct_delegations",
    ):
        return []

    if method in (
        "condenser_api.get_transaction_hex",
        "condenser_api.get_hardfork_version",
        "wallet_bridge_api.get_hardfork_version",
    ):
        return ""

    if method in ("condenser_api.verify_authority",):
        return True

    if method in ("condenser_api.get_account_count","condenser_api.get_witness_count"):
        return 0

    return {}



import os
import json
from arkivist import Arkivist
from sometime import Sometime
from nektar import Waggle
import nektar

def main():

    dt = Sometime().custom("%Y.%m.%d")

    counter = 0
    print("Initializing tests...")
    path = os.path.expanduser("~").replace("\\", "/")
    tests = Arkivist(path + "/nektar-tests.json", autosave=False)

    ##################################################
    # Waggle Class                                   #
    # Methods for blogging and engagement            #
    ##################################################
    
    NEKTAR_VERSION = nektar.__version__
    # custom_nodes = ["testnet.openhive.network"]
    # TESTNET = "18dcf0a285365fc58b71f18b3d3fec954aa0c141c44e4e5cb4cf777b9eab274e"
    # MAINNET = "beeab0de00000000000000000000000000000000000000000000000000000000"

    counter += 1
    print(f"\nTEST #{counter}: Initialize Waggle class with dictionary of WIFs.")
    hive = Waggle(tests["username2"], wifs=tests["wifs2"])

    """
    counter += 1
    print(f"\nTEST #{counter}: Get the list of communities")
    communities = {}
    sorting = ["new", "rank", "subs"]
    for sort in sorting:
        for community in hive.communities(limit=100, sort=sort):
            communities.update({community["name"]: community})
    community = list(communities.keys())[5]
    print(", ".join(list(communities.keys())))

    counter += 1
    print(f"\nTEST #{counter}: Get the name and title of each community")
    print(len(communities))
    for name, data in communities.items():
        print(name + "\t" + data["title"])
        break

    counter += 1
    print(f"\nTEST #{counter}: Get the subscribers of a community.")
    subscribers = {}
    for subscriber in hive.subscribers(community, limit=10):
        subscribers.update({subscriber[0]: subscriber})
        print(subscriber[0])
        break

    counter += 1
    print(f"\nTEST #{counter}: Get the list of 100 Hive blockchain accounts.")
    accounts = {}
    for account in hive.accounts():
        print(account)
        break

    counter += 1
    print(
        f"\nTEST #{counter}: Get the list of 10 Hive blockchain starting with `a`."
    )
    accounts = {}
    for account in hive.accounts(start="a", limit=10):
        print(account)

    counter += 1
    print(
        f"\nTEST #{counter}: Get the list of 10 community posts in order of creation and not yet paid out."
    )
    posts = hive.posts(community, limit=10, sort="created", paidout=False)
    for post in posts:
        print(post["title"])

    counter += 1
    print(
        f"\nTEST #{counter}: Get the list of 1000 followers of the initialized account."
    )
    followers = hive.followers()
    print(", ".join(followers))

    counter += 1
    print(
        f"\nTEST #{counter}: Get the list of 10 muted followers of another account."
    )
    followers = hive.followers(account=tests["username1"], ignore=True, limit=10)
    print(", ".join(followers))

    counter += 1
    print(
        f"\nTEST #{counter}: Get the list of up to 1000 account transactions, most recent first."
    )
    transactions = hive.history()
    print("Transactions:", json.dumps(transactions[1], indent=2))

    counter += 1
    print(
        f"\nTEST #{counter}: Get the list of up to 1000 account transactions of another account."
    )
    transactions = hive.history(account=tests["username1"], start=1000, low=0)
    print("Transactions:", json.dumps(transactions[1], indent=2))

    counter += 1
    print(
        f"\nTEST #{counter}: Get the list of up to 100 account transactions of another account starting from the 5000th transaction to the oldest."
    )
    transactions = hive.history(account=tests["username1"], start=5000, limit=100)
    for transaction in transactions[:1]:
        print(transaction[0], transaction[1]["op"])

    counter += 1
    print(
        f"\nTEST #{counter}: Get the list of posts of the initizalized account."
    )
    blogs = hive.blogs()
    print(json.dumps(blogs[0], indent=2))

    counter += 1
    print(
        f"\nTEST #{counter}: Get the list of posts of another account with non-default values."
    )
    blogs = hive.blogs(account=tests["username1"], sort="created", paidout=True, limit=100)
    print(json.dumps(blogs[0], indent=2))

    counter += 1
    print(f"\nTEST #{counter}: Get the post data.")
    author = tests["username1"]
    permlink = tests["permlink1"]
    data = hive.get_post(author, permlink)
    print(json.dumps(data, indent=2))

    counter += 1
    print(
        f"\nTEST #{counter}: Try to get the post data, return empty if not yet present in the blockchain."
    )
    author = tests["username1"]
    permlink = "abc-123-def-456"
    data = hive.get_post(author, permlink, retries=2)
    print(json.dumps(data, indent=2))
    
    counter += 1
    print(f"\nTEST #{counter}: Publish a new post.")
    dt = Sometime().custom("%Y-%m-%d %H:%M:%S%Z")
    title = f"UPDATE! Connect to Hive with Nektar v{NEKTAR_VERSION} for Python"
    body = f"Introducing Nektar v{NEKTAR_VERSION}! \n\n![](https://images.hive.blog/1536x0/https://images.ecency.com/DQmSYmmPbB9DqXRhgMpC3Ef4NyaF1QJCxm2CqGi9vWpfWTa/honey.jpg)\n<center><sup>Photo by [micheile dot com](https://unsplash.com/photos/lgBPvNjHe6k) on Unsplash</sup></center>\n\n\"How do I connect to Hive?\"\n\nWell, you can now use Nektar.\n\n**[Nektar](https://github.com/rmaniego/nektar)** allows you to to the Hive blockchain using the Hive API.\n\n*Checkout the ReadMe for the full list of features and udpates!*\n\n\n---\n\n\n**TEST DATE:** {dt}\n\n***DISCLAIMER:** This is an official Nektar SDK test, do not flag!!*"
    description = "Nektar is here!"
    tags = "nektar hive api coderundebug"
    community = tests["community2"]
    result = hive.new_post(title, body, description, tags, community, mock=True)
    
    counter += 1
    print(f"\nTEST #{counter}: Reply a comment to a post or another comment.")
    author = tests["username1"]
    permlink = tests["permlink1"]
    body = "Great work in there, keep it up!\n\n---\n" "Check our [community](#) page!\n\n***DISCLAIMER:**This is an official Nektar SDK test only, do not flag!!*"
    hive.reply(author, permlink, body, mock=True)

    counter += 1
    print(f"\nTEST #{counter}: Vote to a post or another comment.")
    author = tests["username1"]
    permlink = tests["permlink1"]
    weight = 100
    hive.vote(author, permlink, weight, mock=True)

    counter += 1
    print(f"\nTEST #{counter}: Get a recently signed transaction.")
    signed_transaction = hive.appbase.signed_transaction

    counter += 1
    # Passing on MAINNET only!
    print(
        f"\nTEST #{counter}: Verify if signed transaction contains required auths."
    )
    verified = hive.verify_authority(signed_transaction, mock=True)
    print("OK: " + str(verified))
    """
    
    counter += 1
    print(f"\nTEST #{counter}: Power Up")
    hive.power_up(tests["username2"], 0.001, mock=True)


if __name__ == "__main__":
    main()
